﻿{
   openjp2.dll unit for Delphi
   original : OPENJPEG LIBRARY ( http://http://www.openjpeg.org/ )
              PasJpeg2000 v1.00 ( http://galfar.vevb.net/wp/projects/jpeg2000-for-pascal/ )

   rev. 1
   date 2014/11/06
   auth ICHIMARU Takeshi ( http://homepage1.nifty.com/Ayakawa/ )
   memo convert from openjpeg.h (in openjpeg-2.1.0.tar.gz)
        and, cut & paste from OpenJpeg.pas (ver 1.03)

   rev. 2
   date 2014/11/07
   auth ICHIMARU Takeshi ( http://homepage1.nifty.com/Ayakawa/ )
   memo stop delayed loading, change to use LoadLibrary/FreeLibrary.
        but if you want to use delayed, define OPENJP2_USE_DELAYED

   rev. 3
   date 2016/10/27
   auth ICHIMARU Takeshi( http://ayakawa.o.oo7.jp )
   memo from OpenJPEG ver 2.1.2

   rev. 4
   date 2017/08/10
   auth ICHIMARU Takeshi( http://ayakawa.o.oo7.jp )
   memo bug fix : LoadOpenJp2Dll func always return FALSE
        support OpenJpeg v2.2.0

   rev. 5
   date 2017/10/05
   auth ICHIMARU Takeshi( http://ayakawa.o.oo7.jp )
   memo support OpenJpeg v2.3.0

   rev. 6
   date 2021/01/10
   auth ICHIMARU Takeshi( http://ayakawa.o.oo7.jp )
   memo support OpenJpeg v2.4.0

   rev. 7
   date 2022/05/15
   auth ICHIMARU Takeshi( http://ayakawa.o.oo7.jp )
   memo support OpenJpeg v2.5.0
}

{*
 * The copyright in this software is being made available under the 2-clauses
 * BSD License, included below. This software may be subject to other third
 * party and contributor rights, including patent rights, and no such rights
 * are granted under this license.
 *
 * Copyright (c) 2014-2022, ICHIMARU Takeshi
 * Copyright (c) 2002-2014, Universite catholique de Louvain (UCL), Belgium
 * Copyright (c) 2002-2014, Professor Benoit Macq
 * Copyright (c) 2001-2003, David Janssens
 * Copyright (c) 2002-2003, Yannick Verschueren
 * Copyright (c) 2003-2007, Francois-Olivier Devaux
 * Copyright (c) 2003-2014, Antonin Descampe
 * Copyright (c) 2005, Herve Drolon, FreeImage Team
 * Copyright (c) 2006-2007, Parvatha Elangovan
 * Copyright (c) 2008, Jerome Fimes, Communications & Systemes <jerome.fimes@c-s.fr>
 * Copyright (c) 2010-2011, Kaori Hagihara
 * Copyright (c) 2011-2012, Centre National d'Etudes Spatiales (CNES), France
 * Copyright (c) 2012, CS Systemes d'Information, France
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS `AS IS'
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *}

unit OpenJpeg21;

{$DEFINE DCC}
{$ALIGN 8}
{$MINENUMSIZE 4}

(* if you want to use delayed loading, enabled OPENJP2_USE_DELAYED.
{$DEFINE OPENJP2_USE_DELAYED}
 *)

interface

uses
  Winapi.windows,
  System.sysutils,
  OpenJpeg21Fncs;

const
{$IFDEF WIN64}
  OpenJpegDllName  = 'openjp2.dll';
{$ELSE}
  OpenJpegDllName  = 'openjp2.dll';
{$ENDIF}
  OPENJPEG_VERSION = '2.5.0';

  OPJ_TRUE         = 1;
  OPJ_FALSE        = 0;

  OPJ_PATH_LEN = 4096;   //**< Maximum allowed size for filenames */

  OPJ_J2K_MAXRLVLS = 33; //**< Number of maximum resolution level authorized */
  OPJ_J2K_MAXBANDS = (3*OPJ_J2K_MAXRLVLS-2);	//**< Number of maximum sub-band linked to number of resolution level */

  OPJ_J2K_DEFAULT_NB_SEGS        =      10;
  OPJ_J2K_STREAM_CHUNK_SIZE      = $100000; //** 1 mega by default */
  OPJ_J2K_DEFAULT_HEADER_SIZE    =    1000;
  OPJ_J2K_MCC_DEFAULT_NB_RECORDS =      10;
  OPJ_J2K_MCT_DEFAULT_NB_RECORDS =      10;

  //* NOT YET USED IN THE V2 VERSION OF OPENJPEG */
  JPWL_MAX_NO_TILESPECS    = 16; //**< Maximum number of tile parts expected by JPWL: increase at your will */
  JPWL_MAX_NO_PACKSPECS    = 16; //**< Maximum number of packet parts expected by JPWL: increase at your will */
  JPWL_MAX_NO_MARKERS      = 512; //**< Maximum number of JPWL markers: increase at your will */
  JPWL_PRIVATEINDEX_NAME   = 'jpwl_index_privatefilename'; //**< index file name used when JPWL is on */
  JPWL_EXPECTED_COMPONENTS = 3; //**< Expect this number of components, so you'll find better the first EPB */
  JPWL_MAXIMUM_TILES       = 8192; //**< Expect this maximum number of tiles, to avoid some crashes */
  JPWL_MAXIMUM_HAMMING     = 2; //**< Expect this maximum number of bit errors in marker id's */
  JPWL_MAXIMUM_EPB_ROOM    = 65450; //**< Expect this maximum number of bytes for composition of EPBs */

{**
 * EXPERIMENTAL FOR THE MOMENT
 * Supported options about file information used only in j2k_dump
 *}
  OPJ_IMG_INFO     =   1; //**< Basic image information provided to the user */
  OPJ_J2K_MH_INFO  =   2; //**< Codestream information based only on the main header */
  OPJ_J2K_TH_INFO  =   4; //**< Tile information based on the current tile header */
  OPJ_J2K_TCH_INFO =   8; //**< Tile/Component information of all tiles */
  OPJ_J2K_MH_IND   =  16; //**< Codestream index based only on the main header */
  OPJ_J2K_TH_IND   =  32; //**< Tile index based on the current tile */
  //*FIXME #define OPJ_J2K_CSTR_IND	48*/	/**<  */
  OPJ_JP2_INFO     = 128; //**< JP2 file information */
  OPJ_JP2_IND      = 256; //**< JP2 file index */

{**
 * JPEG 2000 Profiles, see Table A.10 from 15444-1 (updated in various AMD)
 * These values help chosing the RSIZ value for the J2K codestream.
 * The RSIZ value triggers various encoding options, as detailed in Table A.10.
 * If OPJ_PROFILE_PART2 is chosen, it has to be combined with one or more extensions
 * described hereunder.
 *   Example: rsiz = OPJ_PROFILE_PART2 | OPJ_EXTENSION_MCT;
 * For broadcast profiles, the OPJ_PROFILE value has to be combined with the targeted
 * mainlevel (3-0 LSB, value between 0 and 11):
 *   Example: rsiz = OPJ_PROFILE_BC_MULTI | 0x0005; (here mainlevel 5)
 * For IMF profiles, the OPJ_PROFILE value has to be combined with the targeted mainlevel
 * (3-0 LSB, value between 0 and 11) and sublevel (7-4 LSB, value between 0 and 9):
 *   Example: rsiz = OPJ_PROFILE_IMF_2K | 0x0040 | 0x0005; (here main 5 and sublevel 4)
 **}
  OPJ_PROFILE_NONE       = $0000; //** no profile, conform to 15444-1 */
  OPJ_PROFILE_0          = $0001; //** Profile 0 as described in 15444-1,Table A.45 */
  OPJ_PROFILE_1          = $0002; //** Profile 1 as described in 15444-1,Table A.45 */
  OPJ_PROFILE_PART2      = $8000; //** At least 1 extension defined in 15444-2 (Part-2) */
  OPJ_PROFILE_CINEMA_2K  = $0003; //** 2K cinema profile defined in 15444-1 AMD1 */
  OPJ_PROFILE_CINEMA_4K  = $0004; //** 4K cinema profile defined in 15444-1 AMD1 */
  OPJ_PROFILE_CINEMA_S2K = $0005; //** Scalable 2K cinema profile defined in 15444-1 AMD2 */
  OPJ_PROFILE_CINEMA_S4K = $0006; //** Scalable 4K cinema profile defined in 15444-1 AMD2 */
  OPJ_PROFILE_CINEMA_LTS = $0007; //** Long term storage cinema profile defined in 15444-1 AMD2 */
  OPJ_PROFILE_BC_SINGLE  = $0100; //** Single Tile Broadcast profile defined in 15444-1 AMD3 */
  OPJ_PROFILE_BC_MULTI   = $0200; //** Multi Tile Broadcast profile defined in 15444-1 AMD3 */
  OPJ_PROFILE_BC_MULTI_R = $0300; //** Multi Tile Reversible Broadcast profile defined in 15444-1 AMD3 */
  OPJ_PROFILE_IMF_2K     = $0400; //** 2K Single Tile Lossy IMF profile defined in 15444-1 AMD 8 */
{ These values are changed by v2.4.0
  OPJ_PROFILE_IMF_4K     = $0401; //** 4K Single Tile Lossy IMF profile defined in 15444-1 AMD 8 */
  OPJ_PROFILE_IMF_8K     = $0402; //** 8K Single Tile Lossy IMF profile defined in 15444-1 AMD 8 */
  OPJ_PROFILE_IMF_2K_R   = $0403; //** 2K Single/Multi Tile Reversible IMF profile defined in 15444-1 AMD 8 */
}
  OPJ_PROFILE_IMF_4K     = $0500; //** 4K Single Tile Lossy IMF profile defined in 15444-1 AMD 8 */
  OPJ_PROFILE_IMF_8K     = $0600; //** 8K Single Tile Lossy IMF profile defined in 15444-1 AMD 8 */
  OPJ_PROFILE_IMF_2K_R   = $0700; //** 2K Single/Multi Tile Reversible IMF profile defined in 15444-1 AMD 8 */
  OPJ_PROFILE_IMF_4K_R   = $0800; //** 4K Single/Multi Tile Reversible IMF profile defined in 15444-1 AMD 8 */
{ This value is changed by v2.4.0
  OPJ_PROFILE_IMF_8K_R   = $0801; //** 8K Single/Multi Tile Reversible IMF profile defined in 15444-1 AMD 8 */
}
  OPJ_PROFILE_IMF_8K_R   = $0900; //** 8K Single/Multi Tile Reversible IMF profile defined in 15444-1 AMD 8 */

{**
 * JPEG 2000 Part-2 extensions
 **}
  OPJ_EXTENSION_NONE = $0000; //** No Part-2 extension */
  OPJ_EXTENSION_MCT  = $0100; //** Custom MCT support */

{** Max. Components Sampling Rate (MSamples/sec) per IMF main level *}
  OPJ_IMF_MAINLEVEL_1_MSAMPLESEC  =    65;   //** MSamples/sec for IMF main level 1 */
  OPJ_IMF_MAINLEVEL_2_MSAMPLESEC  =   130;   //** MSamples/sec for IMF main level 2 */
  OPJ_IMF_MAINLEVEL_3_MSAMPLESEC  =   195;   //** MSamples/sec for IMF main level 3 */
  OPJ_IMF_MAINLEVEL_4_MSAMPLESEC  =   260;   //** MSamples/sec for IMF main level 4 */
  OPJ_IMF_MAINLEVEL_5_MSAMPLESEC  =   520;   //** MSamples/sec for IMF main level 5 */
  OPJ_IMF_MAINLEVEL_6_MSAMPLESEC  =  1200;   //** MSamples/sec for IMF main level 6 */
  OPJ_IMF_MAINLEVEL_7_MSAMPLESEC  =  2400;   //** MSamples/sec for IMF main level 7 */
  OPJ_IMF_MAINLEVEL_8_MSAMPLESEC  =  4800;   //** MSamples/sec for IMF main level 8 */
  OPJ_IMF_MAINLEVEL_9_MSAMPLESEC  =  9600;   //** MSamples/sec for IMF main level 9 */
  OPJ_IMF_MAINLEVEL_10_MSAMPLESEC = 19200;   //** MSamples/sec for IMF main level 10 */
  OPJ_IMF_MAINLEVEL_11_MSAMPLESEC = 38400;   //** MSamples/sec for IMF main level 11 */

{** Max. compressed Bit Rate (Mbits/s) per IMF sub level *}
  OPJ_IMF_SUBLEVEL_1_MBITSSEC =     200;     //** Mbits/s for IMF sub level 1 */
  OPJ_IMF_SUBLEVEL_2_MBITSSEC =     400;     //** Mbits/s for IMF sub level 2 */
  OPJ_IMF_SUBLEVEL_3_MBITSSEC =     800;     //** Mbits/s for IMF sub level 3 */
  OPJ_IMF_SUBLEVEL_4_MBITSSEC =    1600;     //** Mbits/s for IMF sub level 4 */
  OPJ_IMF_SUBLEVEL_5_MBITSSEC =    3200;     //** Mbits/s for IMF sub level 5 */
  OPJ_IMF_SUBLEVEL_6_MBITSSEC =    6400;     //** Mbits/s for IMF sub level 6 */
  OPJ_IMF_SUBLEVEL_7_MBITSSEC =   12800;     //** Mbits/s for IMF sub level 7 */
  OPJ_IMF_SUBLEVEL_8_MBITSSEC =   25600;     //** Mbits/s for IMF sub level 8 */
  OPJ_IMF_SUBLEVEL_9_MBITSSEC =   51200;     //** Mbits/s for IMF sub level 9 */

{**
 * JPEG 2000 codestream and component size limits in cinema profiles
 **}
  OPJ_CINEMA_24_CS   = 1302083; //** Maximum codestream length for 24fps */
  OPJ_CINEMA_48_CS   =  651041; //** Maximum codestream length for 48fps */
  OPJ_CINEMA_24_COMP = 1041666; //** Maximum size per color component for 2K & 4K @ 24fps */
  OPJ_CINEMA_48_COMP =  520833; //** Maximum size per color component for 2K @ 48fps */


  OPJ_DPARAMETERS_IGNORE_PCLR_CMAP_CDEF_FLAG = $0001;
  OPJ_DPARAMETERS_DUMP_FLAG                  = $0002;

{**
 * Stream open flags.
 **}
  //** The stream was opened for reading. */
  OPJ_STREAM_READ  = OPJ_TRUE;
  //** The stream was opened for writing. */
  OPJ_STREAM_WRITE = OPJ_FALSE;

type
  EOpenJp2Error = class(Exception);

  OPJ_CHAR      = AnsiChar;  // char
  OPJ_FLOAT32   = single;    // float
  POPJ_FLOAT32  = ^OPJ_FLOAT32;
  OPJ_FLOAT64   = double;    // double
  OPJ_BYTE      = BYTE;      //  8bit UInt
  POPJ_BYTE     = ^OPJ_BYTE;
  OPJ_INT8      = shortint;  //  8bit Int
  OPJ_UINT8     = Byte;      //  8bit UInt
  OPJ_INT16     = SmallInt;  // 16bit Int
  OPJ_UINT16    = WORD;      // 16bit UInt
  OPJ_INT32     = integer;   // 32bit Int
  POPJ_INT32    = ^OPJ_INT32;
  OPJ_UINT32    = Cardinal;  // 32bit UInt
  POPJ_UINT32   = ^OPJ_UINT32;
  OPJ_INT64     = Int64;     // 64bit Int
  OPJ_UINT64    = UInt64;    // 64bit UInt
  OPJ_BOOL      = Int32;     // 32bit Int
  POPJ_BOOL     = ^OPJ_BOOL;
  OPJ_OFF_T     = Int64;     // 64-bit file offset type
  OPJ_SIZE_T    = NativeInt; // WIN32=32bit/WIN64=64bit

  POPJ_STREAM_T = pointer;

{==========================================================
    enum definitions
 ==========================================================}

{**
 * DEPRECATED: use RSIZ, OPJ_PROFILE_* and OPJ_EXTENSION_* instead
 * Rsiz Capabilities
 **}
  opj_RSIZ_CAPABILITIES = (
    OPJ_STD_RSIZ = 0, { Standard JPEG2000 profile }
    OPJ_CINEMA2K = 3, { Profile name for a 2K image }
    OPJ_CINEMA4K = 4, { Profile name for a 4K image }
    OPJ_MCT      = $8100
  );

{**
 * DEPRECATED: use RSIZ, OPJ_PROFILE_* and OPJ_EXTENSION_* instead
 * Digital cinema operation mode
 **}
  opj_CINEMA_MODE = (
    OPJ_OFF         = 0,  { Not Digital Cinema }
    OPJ_CINEMA2K_24 = 1,  { 2K Digital Cinema at 24 fps }
    OPJ_CINEMA2K_48 = 2,  { 2K Digital Cinema at 48 fps }
    OPJ_CINEMA4K_24 = 3   { 4K Digital Cinema at 24 fps }
  );

{**
 * Progression order
 **}
  opj_PROG_ORDER = (
    OPJ_PROG_UNKNOWN = -1, { place-holder }
    OPJ_LRCP         =  0, { layer-resolution-component-precinct order }
    OPJ_RLCP         =  1, { resolution-layer-component-precinct order }
    OPJ_RPCL         =  2, { resolution-precinct-component-layer order }
    OPJ_PCRL         =  3, { precinct-component-resolution-layer order }
    OPJ_CPRL         =  4  { component-precinct-resolution-layer order }
  );

  { Supported image color spaces }
  opj_COLOR_SPACE = (
    OPJ_CLRSPC_UNKNOWN     = -1, { not supported by the library }
    OPJ_CLRSPC_UNSPECIFIED =  0, { not specified in the codestream }
    OPJ_CLRSPC_SRGB        =  1, { sRGB }
    OPJ_CLRSPC_GRAY        =  2, { grayscale }
    OPJ_CLRSPC_SYCC        =  3, { YUV }
    OPJ_CLRSPC_EYCC        =  4, { e-YCC }
    OPJ_CLRSPC_CMYK        =  5  { CMYK }
  );

{**
 * Supported codec
 *}
  opj_CODEC_FORMAT = (
    OPJ_CODEC_UNKNOWN = -1, { place-holder }
    OPJ_CODEC_J2K     =  0, { JPEG-2000 codestream : read/write }
    OPJ_CODEC_JPT     =  1, { JPT-stream (JPEG 2000, JPIP) : read only }
    OPJ_CODEC_JP2     =  2, { JP2 file format : read/write }
    OPJ_CODEC_JPP     =  3, { JPP-stream (JPEG 2000, JPIP) : to be coded }
    OPJ_CODEC_JPX     =  4  { JPX file format (JPEG 2000 Part-2) : to be coded }
  );

{==========================================================
   event manager typedef definitions
 ==========================================================}

{**
 * Callback function prototype for events
 * @param msg               Event message
 * @param client_data       Client object where will be return the event message
 **}
  TOPJ_MSG_CALLBACK = procedure(msg:PAnsiChar; client_data:pointer); cdecl;
  POPJ_MSG_CALLBACK = ^TOPJ_MSG_CALLBACK;


{==========================================================
   codec typedef definitions
 ==========================================================}

  { Progression order changes }
  opj_poc = record
    resno0,                 { Resolution num start, Component num start, given by POC }
    compno0 : OPJ_UINT32;
    layno1,                 { Layer num end,Resolution num end, Component num end, given by POC }
    resno1,
    compno1 : OPJ_UINT32;
    layno0,                 { Layer num start,Precinct num start, Precinct num end }
    precno0,
    precno1 : OPJ_UINT32;
    prg1,
    prg : OPJ_PROG_ORDER;   { Progression order enum }
    progorder : array[0..4] of OPJ_CHAR; { Progression order string }
    tile : OPJ_UINT32;      { Tile number }
    tx0,                    { Start and end values for Tile width and height }
    tx1,
    ty0,
    ty1 : OPJ_UINT32;
    layS,                   { Start value, initialised in pi_initialise_encode }
    resS,
    compS,
    prcS : OPJ_UINT32;
    layE,                   { End value, initialised in pi_initialise_encode }
    resE,
    compE,
    prcE : OPJ_UINT32;
    txS,                    { Start and end values of Tile width and height, initialised in pi_initialise_encode }
    txE,
    tyS,
    tyE,
    dx,
    dy : OPJ_UINT32;
    lay_t,                  { Temporary values for Tile parts, initialised in pi_create_encode }
    res_t,
    comp_t,
    prc_t,
    tx0_t,
    ty0_t : OPJ_UINT32;
  end;
  opj_poc_t = opj_poc;
  popj_poc_t = ^opj_poc_t;

  { Compression parameters }
  opj_cparameters = record
    //** size of tile: tile_size_on = false (not in argument) or = true (in argument) */
    tile_size_on: OPJ_BOOL;
    //** XTOsiz */
    cp_tx0: Integer;
    //** YTOsiz */
    cp_ty0: Integer;
    //** XTsiz */
    cp_tdx: Integer;
    //** YTsiz */
    cp_tdy: Integer;
    //** allocation by rate/distortion */
    cp_disto_alloc: Integer;
    //** allocation by fixed layer */
    cp_fixed_alloc: Integer;
    //** add fixed_quality */
    cp_fixed_quality: Integer;
    //** fixed layer */
    cp_matrice: PInteger;
    //** comment for coding */
    cp_comment: PAnsiChar;
    //** csty : coding style */
    csty: Integer;
    //** progression order (default OPJ_LRCP) */
    prog_order: opj_PROG_ORDER;
    //** progression order changes */
    POC: array[0..31] of opj_poc_t;
    //** number of progression order changes (POC), default to 0 */
    numpocs: OPJ_UINT32;
    //** number of layers */
    tcp_numlayers: Integer;
    //** rates of layers - might be subsequently limited by the max_cs_size field */
    tcp_rates: array[0..99] of Single;
    //** different psnr for successive layers */
    tcp_distoratio: array[0..99] of Single;
    //** number of resolutions */
    numresolution: Integer;
    //** initial code block width, default to 64 */
    cblockw_init: Integer;
    //** initial code block height, default to 64 */
    cblockh_init: Integer;
    //** mode switch (cblk_style) */
    mode: Integer;
    //** 1 : use the irreversible DWT 9-7, 0 : use lossless compression (default) */
    irreversible: Integer;
    //** region of interest: affected component in [0..3], -1 means no ROI */
    roi_compno: Integer;
    //** region of interest: upshift value */
    roi_shift: Integer;
    //* number of precinct size specifications */
    res_spec: Integer;
    //** initial precinct width */
    prcw_init: array[0..OPJ_J2K_MAXRLVLS - 1] of Integer;
    //** initial precinct height */
    prch_init: array[0..OPJ_J2K_MAXRLVLS - 1] of Integer;


    //**@name command line encoder parameters (not used inside the library) */
    //** input file name */
    infile: array[0..opj_PATH_LEN - 1] of AnsiChar;
    //** output file name */
    outfile: array[0..opj_PATH_LEN - 1] of AnsiChar;
    //** DEPRECATED. Index generation is now handeld with the opj_encode_with_info() function. Set to NULL */
    index_on: Integer;
    //** DEPRECATED. Index generation is now handeld with the opj_encode_with_info() function. Set to NULL */
    index: array[0..opj_PATH_LEN - 1] of AnsiChar;
    //** subimage encoding: origin image offset in x direction */
    image_offset_x0: Integer;
    //** subimage encoding: origin image offset in y direction */
    image_offset_y0: Integer;
    //** subsampling value for dx */
    subsampling_dx: Integer;
    //** subsampling value for dy */
    subsampling_dy: Integer;
    //** input file format 0: PGX, 1: PxM, 2: BMP 3:TIF*/
    decod_format: Integer;
    //** output file format 0: J2K, 1: JP2, 2: JPT */
    cod_format: Integer;


    //* UniPG>> */ /* NOT YET USED IN THE V2 VERSION OF OPENJPEG */
    //** enables writing of EPC in MH, thus activating JPWL */
    jpwl_epc_on: Bool;
    //** error protection method for MH (0,1,16,32,37-128) */
    jpwl_hprot_MH: Integer;
    //** tile number of header protection specification (>=0) */
    jpwl_hprot_TPH_tileno: array[0..JPWL_MAX_NO_TILESPECS - 1] of Integer;
    //** error protection methods for TPHs (0,1,16,32,37-128) */
    jpwl_hprot_TPH: array[0..JPWL_MAX_NO_TILESPECS - 1] of Integer;
    //** tile number of packet protection specification (>=0) */
    jpwl_pprot_tileno: array[0..JPWL_MAX_NO_PACKSPECS - 1] of Integer;
    //** packet number of packet protection specification (>=0) */
    jpwl_pprot_packno: array[0..JPWL_MAX_NO_PACKSPECS - 1] of Integer;
    //** error protection methods for packets (0,1,16,32,37-128) */
    jpwl_pprot: array[0..JPWL_MAX_NO_PACKSPECS - 1] of Integer;
    //** enables writing of ESD, (0=no/1/2 bytes) */
    jpwl_sens_size: Integer;
    //** sensitivity addressing size (0=auto/2/4 bytes) */
    jpwl_sens_addr: Integer;
    //** sensitivity range (0-3) */
    jpwl_sens_range: Integer;
    //** sensitivity method for MH (-1=no,0-7) */
    jpwl_sens_MH: Integer;
    //** tile number of sensitivity specification (>=0) */
    jpwl_sens_TPH_tileno: array[0..JPWL_MAX_NO_TILESPECS - 1] of Integer;
    //** sensitivity methods for TPHs (-1=no,0-7) */
    jpwl_sens_TPH: array[0..JPWL_MAX_NO_TILESPECS - 1] of Integer;


    {**
     * DEPRECATED: use RSIZ, OPJ_PROFILE_* and MAX_COMP_SIZE instead
     * Digital Cinema compliance 0-not compliant, 1-compliant
     **}
    cp_cinema: opj_CINEMA_MODE;
    {**
     * Maximum size (in bytes) for each component.
     * If == 0, component size limitation is not considered
     **}
    max_comp_size: Integer;
    {**
     * DEPRECATED: use RSIZ, OPJ_PROFILE_* and OPJ_EXTENSION_* instead
     * Profile name
     **}
    cp_rsiz: opj_RSIZ_CAPABILITIES;
    //** Tile part generation*/
    tp_on: Byte;
    //** Flag for Tile part generation*/
    tp_flag: Byte;
    //** MCT (multiple component transform) */
    tcp_mct: Byte;
    //** Enable JPIP indexing*/
    jpip_on: OPJ_BOOL;
    {** Naive implementation of MCT restricted to a single reversible array based
        encoding without offset concerning all the components. *}
    mct_data: pointer;
    {**
     * Maximum size (in bytes) for the whole codestream.
     * If == 0, codestream size limitation is not considered
     * If it does not comply with tcp_rates, max_cs_size prevails
     * and a warning is issued.
     **}
    max_cs_size: integer;
    {** RSIZ value
        To be used to combine OPJ_PROFILE_*, OPJ_EXTENSION_* and (sub)levels values. *}
    rsiz: OPJ_UINT16;
  end;
  opj_cparameters_t = opj_cparameters;
  popj_cparameters_t = ^opj_cparameters_t;


{**
 * Decompression parameters
 **}
  opj_dparameters = record
    {**
      Set the number of highest resolution levels to be discarded.
      The image resolution is effectively divided by 2 to the power of the number of discarded levels.
      The reduce factor is limited by the smallest total number of decomposition levels among tiles.
      if != 0, then original dimension divided by 2^(reduce);
      if == 0 or not used, image is decoded to the full resolution
    **}
    cp_reduce: OPJ_UINT32;
    {**
      Set the maximum number of quality layers to decode.
      If there are less quality layers than the specified number, all the quality layers are decoded.
      if != 0, then only the first "layer" layers are decoded;
      if == 0 or not used, all the quality layers are decoded
     **}
    cp_layer: OPJ_UINT32;

    {**@name command line decoder parameters (not used inside the library) *}
    {** input file name *}
    infile: array[0..opj_PATH_LEN - 1] of AnsiChar;
    {** output file name *}
    outfile: array[0..opj_PATH_LEN - 1] of AnsiChar;
    {** input file format 0: J2K, 1: JP2, 2: JPT *}
    decod_format: Integer;
    {** output file format 0: PGX, 1: PxM, 2: BMP *}
    cod_format: Integer;

    {** Decoding area left boundary *}
    DA_x0: OPJ_UINT32;
    {** Decoding area right boundary *}
    DA_x1: OPJ_UINT32;
    {** Decoding area up boundary *}
    DA_y0: OPJ_UINT32;
    {** Decoding area bottom boundary *}
    DA_y1: OPJ_UINT32;
    {** Verbose mode *}
    m_verbose: OPJ_BOOL;

    {** tile number ot the decoded tile*}
    tile_index: OPJ_UINT32;
    {** Nb of tile to decode *}
    nb_tile_to_decode: OPJ_UINT32;

    {* NOT YET USED IN THE V2 VERSION OF OPENJPEG *}
    jpwl_correct: Bool;                       //** activates the JPWL correction capabilities */
    jpwl_exp_comps: Integer;                  //** expected number of components */
    jpwl_max_tiles: Integer;                  //** maximum number of tiles */

    flags: DWORD;
  end;
  opj_dparameters_t = opj_dparameters;
  popj_dparameters_t = ^opj_dparameters_t;

{*
==========================================================
   I/O stream typedef definitions
==========================================================
*}

{*
 * Callback function prototype for read function
 *}
  opj_stream_read_fn = function(p_buffer:pointer; p_nb_bytes:OPJ_SIZE_T; p_user_data:pointer):OPJ_SIZE_T; cdecl;
  popj_stream_read_fn = ^opj_stream_read_fn;
{*
 * Callback function prototype for write function
 *}
  opj_stream_write_fn = function(p_buffer:pointer; p_nb_bytes:OPJ_SIZE_T; p_user_data:pointer):OPJ_SIZE_T; cdecl;
  popj_stream_write_fn = ^opj_stream_write_fn;
{*
 * Callback function prototype for skip function
 *}
  opj_stream_skip_fn = function(p_nb_butes:OPJ_OFF_T; p_user_data:pointer):OPJ_OFF_T; cdecl;
  popj_stream_skip_fn = ^opj_stream_skip_fn;
{*
 * Callback function prototype for seek function
 *}
  opj_stream_seek_fn = function(p_nb_bytes:OPJ_OFF_T; p_user_data:pointer):OPJ_BOOL; cdecl;
  popj_stream_seek_fn = ^opj_stream_seek_fn;
{*
 * Callback function prototype for free user data function
 *}
  opj_stream_free_user_data_fn = procedure(p_user_data:pointer); cdecl;
  popj_stream_free_user_data_fn = ^opj_stream_free_user_data_fn;

  { Defines a single image component }
  opj_image_comp = record
    dx: OPJ_UINT32;            { XRsiz: horizontal separation of a sample of ith component with respect to the reference grid }
    dy: OPJ_UINT32;            { YRsiz: vertical separation of a sample of ith component with respect to the reference grid }
    w: OPJ_UINT32;             { data width }
    h: OPJ_UINT32;             { data height }
    x0: OPJ_UINT32;            { x component offset compared to the whole image }
    y0: OPJ_UINT32;            { y component offset compared to the whole image }
    prec: OPJ_UINT32;          { precision }
    bpp: OPJ_UINT32;           { image depth in bits }
    sgnd: OPJ_UINT32;          { signed (1) / unsigned (0) }
    resno_decoded: OPJ_UINT32; { number of decoded resolution }
    factor: OPJ_UINT32;        { number of division by 2 of the out image compared to the original size of image }
    data: PIntegerArray;       { image component data }
    alpha: OPJ_UINT16;         { alpha channel }
  end;
  opj_image_comp_t = opj_image_comp;
  popj_image_comp_t = ^opj_image_comp_t;
  opj_image_comp_array = array[0..255] of opj_image_comp_t;
  popj_image_comp_array = ^opj_image_comp_array;

  { Component parameters structure used by the opj_image_create function }
  opj_image_comptparm = record
    dx: OPJ_UINT32;   { XRsiz: horizontal separation of a sample of ith component with respect to the reference grid }
    dy: OPJ_UINT32;   { YRsiz: vertical separation of a sample of ith component with respect to the reference grid }
    w: OPJ_UINT32;    { data width }
    h: OPJ_UINT32;    { data height }
    x0: OPJ_UINT32;   { x component offset compared to the whole image }
    y0: OPJ_UINT32;   { y component offset compared to the whole image }
    prec: OPJ_UINT32; { precision }
    bpp: OPJ_UINT32;  { image depth in bits }
    sgnd: OPJ_UINT32; { signed (1) / unsigned (0) }
  end;
  opj_image_cmptparm_t = opj_image_comptparm;
  popj_image_cmptparm_t = ^opj_image_cmptparm_t;
  opj_image_cmptparm_array = array[0..255] of opj_image_cmptparm_t;
  popj_image_cmptparm_array = ^opj_image_cmptparm_array;

  { Defines image data and Characteristics }
  opj_image = record
    x0: OPJ_UINT32;               { XOsiz: horizontal offset from the origin of the reference grid to the left side of the image area }
    y0: OPJ_UINT32;               { YOsiz: vertical offset from the origin of the reference grid to the top side of the image area }
    x1: OPJ_UINT32;               { Xsiz: width of the reference grid }
    y1: OPJ_UINT32;               { Ysiz: height of the reference grid }
    numcomps: OPJ_UINT32;         { number of components in the image }
    color_space: opj_COLOR_SPACE; { color space: sRGB, Greyscale or YUV }
    comps: popj_image_comp_array; { image components }
    icc_profile_buf: POPJ_BYTE;   { 'restricted' ICC profile }
    icc_profile_len: OPJ_UINT32;  { size of ICC profile }
  end;
  opj_image_t = opj_image;
  popj_image_t = ^opj_image_t;
  ppopj_image_t = ^popj_image_t;

{**
 * JPEG 2000 profile macros
 **}
{
#define OPJ_IS_CINEMA(v)     (((v) >= OPJ_PROFILE_CINEMA_2K)&&((v) <= OPJ_PROFILE_CINEMA_S4K))
#define OPJ_IS_STORAGE(v)    ((v) == OPJ_PROFILE_CINEMA_LTS)
#define OPJ_IS_BROADCAST(v)  (((v) >= OPJ_PROFILE_BC_SINGLE)&&((v) <= ((OPJ_PROFILE_BC_MULTI_R) | (0x000b))))
#define OPJ_IS_IMF(v)        (((v) >= OPJ_PROFILE_IMF_2K)&&((v) <= ((OPJ_PROFILE_IMF_8K_R) | (0x009b))))
#define OPJ_IS_PART2(v)      ((v) & OPJ_PROFILE_PART2)

#define OPJ_GET_IMF_PROFILE(v)   ((v) & 0xff00)      /** Extract IMF profile without mainlevel/sublevel */
#define OPJ_GET_IMF_MAINLEVEL(v) ((v) & 0xf)         /** Extract IMF main level */
#define OPJ_GET_IMF_SUBLEVEL(v)  (((v) >> 4) & 0xf)  /** Extract IMF sub level */
}

const
  OPJ_IMF_MAINLEVEL_MAX =  11;   //** Maximum main level */

{*
==========================================================
   Information on the JPEG 2000 codestream
==========================================================
*}
//* QUITE EXPERIMENTAL FOR THE MOMENT */

{**
 * Index structure : Information concerning a packet inside tile
 **}
type
  opj_packet_info = record
    //** packet start position (including SOP marker if it exists) */
    start_pos  : OPJ_OFF_T;
    //** end of packet header position (including EPH marker if it exists)*/
    end_ph_pos : OPJ_OFF_T;
    //** packet end position */
    end_pos    : OPJ_OFF_T;
    //** packet distorsion */
    disto: double;
  end;
  opj_packet_info_t = opj_packet_info;
  popj_packet_info_t = ^opj_packet_info_t;

{**
 * Marker structure
 **}
  opj_marker_info = record
    //** marker type */
	  _type: WORD;
    //** position in codestream */
    pos: OPJ_OFF_T;
    //** length, marker val included */
    len: integer;
  end;
  opj_marker_info_t = opj_marker_info;
  popj_marker_info_t = ^opj_marker_info_t;

{**
 * Index structure : Information concerning tile-parts
 *}
  opj_tp_info = record
    //** start position of tile part */
    tp_start_pos: integer;
    //** end position of tile part header */
    tp_end_header: integer;
    //** end position of tile part */
    tp_end_pos: integer;
    //** start packet of tile part */
    tp_start_pack: integer;
    //** number of packets of tile part */
    tp_numpacks: integer;
  end;
  opj_tp_info_t = opj_tp_info;
  popj_tp_info_t = ^opj_tp_info_t;

{**
 * Index structure : information regarding tiles
 *}
  opj_tile_info = record
    //** value of thresh for each layer by tile cfr. Marcela   */
    thresh: pdouble;
    //** number of tile */
    tileno: integer;
    //** start position */
    start_pos: integer;
    //** end position of the header */
    end_header: integer;
    //** end position */
    end_pos: integer;
    //** precinct number for each resolution level (width) */
    pw: array[0..32] of integer;
    //** precinct number for each resolution level (height) */
    ph: array[0..32] of integer;
    //** precinct size (in power of 2), in X for each resolution level */
    pdx: array[0..32] of integer;
    //** precinct size (in power of 2), in Y for each resolution level */
    pdy: array[0..32] of integer;
    //** information concerning packets inside tile */
    packet: popj_packet_info_t;
    //** add fixed_quality */
    numpix: integer;
    //** add fixed_quality */
    distotile: double;
    //** number of markers */
    marknum: integer;
    //** list of markers */
    marker: popj_marker_info_t;
    //** actual size of markers array */
    maxmarknum: integer;
    //** number of tile parts */
    num_tps: integer;
    //** information concerning tile parts */
    tp: popj_tp_info_t;
  end;
  opj_tile_info_t = opj_tile_info;
  popj_tile_info_t = ^opj_tile_info_t;

{**
 * Index structure of the codestream
 *}
  opj_codestream_info = record
    //** maximum distortion reduction on the whole image (add for Marcela) */
    D_max: double;
    //** packet number */
    packno: integer;
    //** writing the packet in the index with t2_encode_packets */
    index_write: integer;
    //** image width */
    image_w: integer;
    //** image height */
    image_h: integer;
    //** progression order */
    prog: OPJ_PROG_ORDER;
    //** tile size in x */
    tile_x: integer;
    //** tile size in y */
    tile_y: integer;
    //** */
    tile_Ox: integer;
    //** */
    tile_Oy: integer;
    //** number of tiles in X */
    tw: integer;
    //** number of tiles in Y */
    th: integer;
    //** component numbers */
    numcomps: integer;
    //** number of layer */
    numlayers: integer;
    //** number of decomposition for each component */
    numdecompos: PInteger;

    //** number of markers */
    marknum: integer;
    //** list of markers */
    marker: popj_marker_info_t;
    //** actual size of markers array */
    maxmarknum: integer;

    //** main header position */
    main_head_start: integer;
    //** main header position */
    main_head_end: integer;
    //** codestream's size */
    codestream_size: integer;
    //** information regarding tiles inside image */
    tile: popj_tile_info_t;
  end;
  opj_codestream_info_t = opj_codestream_info;
  popj_codestream_info_t = ^opj_codestream_info_t;

{* new output managment of the codestream information and index *}

{**
 * Tile-component coding parameters information
 *}
  opj_tccp_info = record
    //** component index */
    compno: OPJ_UINT32;
    //** coding style */
    csty: OPJ_UINT32;
    //** number of resolutions */
    numresolutions: OPJ_UINT32;
    //** code-blocks width */
    cblkw: OPJ_UINT32;
    //** code-blocks height */
    cblkh: OPJ_UINT32;
    //** code-block coding style */
    cblksty: OPJ_UINT32 ;
    //** discrete wavelet transform identifier */
    qmfbid: OPJ_UINT32;
    //** quantisation style */
    qntsty: OPJ_UINT32;
    //** stepsizes used for quantization */
    stepsizes_mant: array[0..OPJ_J2K_MAXBANDS-1] of OPJ_UINT32;
    //** stepsizes used for quantization */
    stepsizes_expn: array[0..OPJ_J2K_MAXBANDS-1] of OPJ_UINT32;
    //** number of guard bits */
    numgbits: OPJ_UINT32;
    //** Region Of Interest shift */
    roishift: OPJ_INT32;
    //** precinct width */
    prcw: array[0..OPJ_J2K_MAXRLVLS-1] of OPJ_UINT32;
    //** precinct height */
    prch: array[0..OPJ_J2K_MAXRLVLS-1] of OPJ_UINT32;
  end;
  opj_tccp_info_t = opj_tccp_info;
  popj_tccp_info_t = ^opj_tccp_info_t;

{**
 * Tile coding parameters information
 *}
  opj_tile_info_v2 = record
    //** number (index) of tile */
    tileno: integer;
    //** coding style */
    csty: OPJ_UINT32;
    //** progression order */
    prg: OPJ_PROG_ORDER;
    //** number of layers */
    numlayers: OPJ_UINT32 ;
    //** multi-component transform identifier */
    mct: OPJ_UINT32;

    //** information concerning tile component parameters*/
    tccp_info: popj_tccp_info_t;
  end;
  opj_tile_info_v2_t = opj_tile_info_v2;
  popj_tile_info_v2_t = ^opj_tile_info_v2_t;

{**
 * Information structure about the codestream (FIXME should be expand and enhance)
 *}
  opj_codestream_info_v2 = record
    //* Tile info */
    //** tile origin in x = XTOsiz */
    tx0: OPJ_UINT32;
    //** tile origin in y = YTOsiz */
    ty0: OPJ_UINT32;
    //** tile size in x = XTsiz */
    tdx: OPJ_UINT32;
    //** tile size in y = YTsiz */
    tdy: OPJ_UINT32;
    //** number of tiles in X */
    tw: OPJ_UINT32;
    //** number of tiles in Y */
    th: OPJ_UINT32;

    //** number of components*/
    nbcomps: OPJ_UINT32;

    //** Default information regarding tiles inside image */
    m_default_tile_info: opj_tile_info_v2_t ;

    //** information regarding tiles inside image */
    tile_info: popj_tile_info_v2_t; //* FIXME not used for the moment */
  end;
  opj_codestream_info_v2_t = opj_codestream_info_v2;
  popj_codestream_info_v2_t = ^opj_codestream_info_v2_t;

{**
 * Index structure about a tile part
 *}
  opj_tp_index = record
    //** start position */
    start_pos: OPJ_OFF_T;
    //** end position of the header */
    end_header: OPJ_OFF_T;
    //** end position */
    end_pos: OPJ_OFF_T;
  end;
  opj_tp_index_t = opj_tp_index;
  popj_tp_index_t = ^opj_tp_index_t;

{**
 * Index structure about a tile
 *}
  opj_tile_index = record
    //** tile index */
    tileno: OPJ_UINT32;

    //** number of tile parts */
    nb_tps: OPJ_UINT32;
    //** current nb of tile part (allocated)*/
    current_nb_tps: OPJ_UINT32;
    //** current tile-part index */
    current_tpsno: OPJ_UINT32;
    //** information concerning tile parts */
    tp_index: popj_tp_index_t;

    //* NOT USED FOR THE MOMENT IN THE V2 VERSION */
    //** number of markers */
    marknum: OPJ_UINT32;
    //** list of markers */
    marker: popj_marker_info_t;
    //** actual size of markers array */
    maxmarknum: OPJ_UINT32;

    //** packet number */
    nb_packet: OPJ_UINT32;
    //** information concerning packets inside tile */
    packet_index: popj_packet_info_t;
  end;
  opj_tile_index_t = opj_tile_index;
  popj_tile_index_t = ^opj_tile_index_t;

{**
 * Index structure of the codestream (FIXME should be expand and enhance)
 *}
  opj_codestream_index = record
    //** main header start position (SOC position) */
    main_head_start: OPJ_OFF_T;
    //** main header end position (first SOT position) */
    main_head_end: OPJ_OFF_T;

    //** codestream's size */
    codestream_size: OPJ_UINT64;

    //* NOT USED FOR THE MOMENT IN THE V2 VERSION */
    //** number of markers */
    marknum: OPJ_UINT32;
    //** list of markers */
    marker: popj_marker_info_t;
    //** actual size of markers array */
    maxmarknum: OPJ_UINT32;

    //** */
    nb_of_tiles: OPJ_UINT32;
    //** */
    tile_index: popj_tile_index_t; //* FIXME not used for the moment */
  end;
  opj_codestream_index_t = opj_codestream_index;
  popj_codestream_index_t = ^opj_codestream_index_t;

{*
==========================================================
   Metadata from the JP2file
==========================================================
*}

{**
 * Info structure of the JP2 file
 * EXPERIMENTAL FOR THE MOMENT
 *}
  opj_jp2_metadata = record
    not_used: OPJ_INT32;
  end;
  opj_jp2_metadata_t = opj_jp2_metadata;
  popj_jp2_metadata_t = ^opj_jp2_metadata_t;

{**
 * Index structure of the JP2 file
 * EXPERIMENTAL FOR THE MOMENT
 *}
  opj_jp2_index = record
    not_used: OPJ_INT32;
  end;
  opj_jp2_index_t = opj_jp2_index;
  popj_jp2_index_t = ^opj_jp2_index_t;

{**
 * JPEG2000 codec V2.
 **}
  popj_codec_t = pointer;


{$WARN SYMBOL_PLATFORM OFF}

{******************************************************************************
 *
 *                 OpenJpeg Version Functions Definitions
 *
 ******************************************************************************}

function opj_version: PAnsiChar; stdcall;
{$IFDEF OPENJP2_USE_DELAYED}
  external OpenJpegDllName name S_OPJ_VERSION delayed;
{$ENDIF}

{******************************************************************************
 *
 *                      Image Functions Definitions
 *
 ******************************************************************************}

{ Create an image
  @param numcmpts number of components
  @param cmptparms components parameters
  @param clrspc image color space
  @return returns a new image structure if successful, returns NULL otherwise }
function opj_image_create(numcmpts: OPJ_UINT32; cmptparms: popj_image_cmptparm_t;
  clrspc: opj_COLOR_SPACE): popj_image_t; stdcall;
{$IFDEF OPENJP2_USE_DELAYED}
  external OpenJpegDllName name S_OPJ_IMAGE_CREATE delayed;
{$ENDIF}

{ Deallocate any resources associated with an image
  @param image image to be destroyed }
procedure opj_image_destroy(image: popj_image_t); stdcall;
{$IFDEF OPENJP2_USE_DELAYED}
  external OpenJpegDllName name S_OPJ_IMAGE_DESTROY delayed;
{$ENDIF}

{**
 * Creates an image without allocating memory for the image (used in the new version of the library).
 *
 * @param	numcmpts    the number of components
 * @param	cmptparms   the components parameters
 * @param	clrspc      the image color space
 *
 * @return	a new image structure if successful, NULL otherwise.
 *}
function opj_image_tile_create(numcmpts: OPJ_UINT32; cmptparms: popj_image_cmptparm_t; clrspc: OPJ_COLOR_SPACE):popj_image_t;
  stdcall;
{$IFDEF OPENJP2_USE_DELAYED}
  external OpenJpegDllName name S_OPJ_IMAGE_TILE_CREATE delayed;
{$ENDIF}

{**
 * Allocator for opj_image_t->comps[].data
 * To be paired with opj_image_data_free.
 *
 * @param   size    number of bytes to allocate
 *
 * @return  a new pointer if successful, NULL otherwise.
 * @since 2.2.0
*}
function opj_image_data_alloc(size:OPJ_SIZE_T):pointer; stdcall;
{$IFDEF OPENJP2_USE_DELAYED}
  external OpenJpegDllName name S_OPJ_IMAGE_DATA_ALLOC delayed;
{$ENDIF}

{**
 * Destructor for opj_image_t->comps[].data
 * To be paired with opj_image_data_alloc.
 *
 * @param   ptr    Pointer to free
 *
 * @since 2.2.0
*}
procedure opj_image_data_free(ptr:pointer); stdcall;
{$IFDEF OPENJP2_USE_DELAYED}
  external OpenJpegDllName name S_OPJ_IMAGE_DATA_FREE delayed;
{$ENDIF}

{******************************************************************************
 *
 *                       Stream Functions Definitions
 *
 ******************************************************************************}

{ Create an abstract stream. This function does nothing except allocating memory and initializing the abstract stream.

  @param  p_is_input if set to true then the stream will be an input stream, an output stream else.
  @return a stream object.
}
function opj_stream_default_create(p_is_input:OPJ_BOOL):popj_stream_t ; stdcall;
{$IFDEF OPENJP2_USE_DELAYED}
  external OpenJpegDllName name S_OPJ_STREAM_DEFAULT_CREATE delayed;
{$ENDIF}

{ Creates an abstract stream. This function does nothing except allocating memory and initializing the abstract stream.

  @param p_buffer_size  FIXME DOC
  @param p_is_input     if set to true then the stream will be an input stream, an output stream else.

  @return a stream object.
}
function opj_stream_create(p_buffer_size:OPJ_SIZE_T; p_is_input:OPJ_BOOL):popj_stream_t; stdcall;
{$IFDEF OPENJP2_USE_DELAYED}
  external OpenJpegDllName name S_OPJ_STREAM_CREATE delayed;
{$ENDIF}

{ Destroys a stream created by opj_create_stream. This function does NOT close the abstract stream.
  If needed the user must close its own implementation of the stream.

  @param p_stream       the stream to destroy.
}
procedure opj_stream_destroy(p_stream:popj_stream_t); stdcall;
{$IFDEF OPENJP2_USE_DELAYED}
  external OpenJpegDllName name S_OPJ_STREAM_DESTROY delayed;
{$ENDIF}

{**
 * Sets the given function to be used as a read function.
 * @param		p_stream	the stream to modify
 * @param		p_function	the function to use a read function.
 *}
procedure opj_stream_set_read_function(p_stream: popj_stream_t; p_function: opj_stream_read_fn); stdcall;
{$IFDEF OPENJP2_USE_DELAYED}
  external OpenJpegDllName name S_OPJ_STREAM_SET_READ_FUNCTION delayed;
{$ENDIF}

{**
 * Sets the given function to be used as a write function.
 * @param		p_stream	the stream to modify
 * @param		p_function	the function to use a write function.
 *}
procedure opj_stream_set_write_function(p_stream: popj_stream_t; p_function: opj_stream_write_fn); stdcall;
{$IFDEF OPENJP2_USE_DELAYED}
  external OpenJpegDllName name S_OPJ_STREAM_SET_WRITE_FUNCTION delayed;
{$ENDIF}

{**
 * Sets the given function to be used as a skip function.
 * @param		p_stream	the stream to modify
 * @param		p_function	the function to use a skip function.
 *}
procedure opj_stream_set_skip_function(p_stream: popj_stream_t; p_function: opj_stream_skip_fn); stdcall;
{$IFDEF OPENJP2_USE_DELAYED}
  external OpenJpegDllName name S_OPJ_STREAM_SET_SKIP_FUNCTION delayed;
{$ENDIF}

{**
 * Sets the given function to be used as a seek function, the stream is then seekable.
 * @param		p_stream	the stream to modify
 * @param		p_function	the function to use a skip function.
 *}
procedure opj_stream_set_seek_function(p_stream: popj_stream_t; p_function: opj_stream_seek_fn); stdcall;
{$IFDEF OPENJP2_USE_DELAYED}
  external OpenJpegDllName name S_OPJ_STREAM_SET_SEEK_FUNCTION delayed;
{$ENDIF}

{**
 * Sets the given data to be used as a user data for the stream.
 * @param		p_stream	the stream to modify
 * @param		p_data		the data to set.
 * @param		p_function	the function to free p_data when opj_stream_destroy() is called.
 *}
procedure opj_stream_set_user_data(p_stream: popj_stream_t; p_data: pointer; p_function: opj_stream_free_user_data_fn); stdcall;
{$IFDEF OPENJP2_USE_DELAYED}
  external OpenJpegDllName name S_OPJ_STREAM_SET_USER_DATA delayed;
{$ENDIF}

{**
 * Sets the length of the user data for the stream.
 *
 * @param p_stream    the stream to modify
 * @param data_length length of the user_data.
 *}
procedure opj_stream_set_user_data_length(p_stream: popj_stream_t; data_length: OPJ_UINT64); stdcall;
{$IFDEF OPENJP2_USE_DELAYED}
  external OpenJpegDllName name S_OPJ_STREAM_SET_USER_DATA_LENGTH delayed;
{$ENDIF}

{**
 * Create a stream from a file identified with its filename with default parameters (helper function)
 * @param fname             the filename of the file to stream
 * @param p_is_read_stream  whether the stream is a read stream (true) or not (false)
 *}
function opj_stream_create_default_file_stream( fname : PAnsiChar; p_is_read_stream: OPJ_BOOL):popj_stream_t; stdcall;
{$IFDEF OPENJP2_USE_DELAYED}
  external OpenJpegDllName name S_OPJ_STREAM_CREATE_DEFAULT_FILE_STREAM delayed;
{$ENDIF}

{** Create a stream from a file identified with its filename with a specific buffer size
 * @param fname             the filename of the file to stream
 * @param p_buffer_size     size of the chunk used to stream
 * @param p_is_read_stream  whether the stream is a read stream (true) or not (false)
 *}
function opj_stream_create_file_stream( fname: PAnsiChar; p_buffer_size: OPJ_SIZE_T;
                                        p_is_read_stream: OPJ_BOOL):popj_stream_t; stdcall;
{$IFDEF OPENJP2_USE_DELAYED}
  external OpenJpegDllName name S_OPJ_STREAM_CREATE_FILE_STREAM delayed;
{$ENDIF}

{******************************************************************************
 *
 *                   event manager functions definitions
 *
 ******************************************************************************}
{**
 * Set the info handler use by openjpeg.
 * @param p_codec       the codec previously initialise
 * @param p_callback    the callback function which will be used
 * @param p_user_data   client object where will be returned the message
*}
function opj_set_info_handler( p_codec: popj_codec_t; p_callback: topj_msg_callback;
                               p_user_data: pointer):OPJ_BOOL; stdcall;
{$IFDEF OPENJP2_USE_DELAYED}
  external OpenJpegDllName name S_OPJ_SET_INFO_HANDLER delayed;
{$ENDIF}

{**
 * Set the warning handler use by openjpeg.
 * @param p_codec       the codec previously initialise
 * @param p_callback    the callback function which will be used
 * @param p_user_data   client object where will be returned the message
*}
function opj_set_warning_handler( p_codec: popj_codec_t; p_callback: topj_msg_callback;
                                  p_user_data: pointer):OPJ_BOOL; stdcall;
{$IFDEF OPENJP2_USE_DELAYED}
  external OpenJpegDllName name S_OPJ_SET_WARNING_HANDLER delayed;
{$ENDIF}

{**
 * Set the error handler use by openjpeg.
 * @param p_codec       the codec previously initialise
 * @param p_callback    the callback function which will be used
 * @param p_user_data   client object where will be returned the message
 *}
function opj_set_error_handler( p_codec: popj_codec_t; p_callback: topj_msg_callback;
                                p_user_data: pointer):OPJ_BOOL; stdcall;
{$IFDEF OPENJP2_USE_DELAYED}
  external OpenJpegDllName name S_OPJ_SET_ERROR_HANDLER delayed;
{$ENDIF}

{******************************************************************************
 *
 *                      codec functions definitions
 *
 ******************************************************************************}

{**
 * Creates a J2K/JP2 decompression structure
 * @param format 		Decoder to select
 *
 * @return Returns a handle to a decompressor if successful, returns NULL otherwise
 **}
function opj_create_decompress(format: OPJ_CODEC_FORMAT):POPJ_CODEC_T; stdcall;
{$IFDEF OPENJP2_USE_DELAYED}
  external OpenJpegDllName name S_OPJ_CREATE_DECOMPRESS delayed;
{$ENDIF}

{**
 * Destroy a decompressor handle
 *
 * @param	p_codec			decompressor handle to destroy
 *}
procedure opj_destroy_codec(p_codec: popj_codec_t); stdcall;
{$IFDEF OPENJP2_USE_DELAYED}
  external OpenJpegDllName name S_OPJ_DESTROY_CODEC delayed;
{$ENDIF}

{**
 * Read after the codestream if necessary
 * @param	p_codec			the JPEG2000 codec to read.
 * @param	p_stream		the JPEG2000 stream.
 *}
function opj_end_decompress( p_codec: popj_codec_t; p_stream: popj_stream_t):OPJ_BOOL; stdcall;
{$IFDEF OPENJP2_USE_DELAYED}
  external OpenJpegDllName name S_OPJ_END_DECOMPRESS delayed;
{$ENDIF}

{**
 * Set decoding parameters to default values
 * @param parameters Decompression parameters
 *}
procedure opj_set_default_decoder_parameters( parameters: popj_dparameters_t); stdcall;
{$IFDEF OPENJP2_USE_DELAYED}
  external OpenJpegDllName name S_OPJ_SET_DEFAULT_DECODER_PARAMETERS delayed;
{$ENDIF}

{**
 * Setup the decoder with decompression parameters provided by the user and with the message handler
 * provided by the user.
 *
 * @param p_codec 		decompressor handler
 * @param parameters 	decompression parameters
 *
 * @return true			if the decoder is correctly set
 *}
function opj_setup_decoder( p_codec: popj_codec_t; parameters: popj_dparameters_t):OPJ_BOOL; stdcall;
{$IFDEF OPENJP2_USE_DELAYED}
  external OpenJpegDllName name S_OPJ_SETUP_DECODER delayed;
{$ENDIF}

{**
 * Set strict decoding parameter for this decoder.  If strict decoding is enabled, partial bit
 * streams will fail to decode.  If strict decoding is disabled, the decoder will decode partial
 * bitstreams as much as possible without erroring
 *
 * @param p_codec       decompressor handler
 * @param strict        OPJ_TRUE to enable strict decoding, OPJ_FALSE to disable
 *
 * @return true         if the decoder is correctly set
 *}

function opj_decoder_set_strict_mode( p_codec : popj_codec_t; strict:OPJ_BOOL):OPJ_BOOL; stdcall;
{$IFDEF OPENJP2_USE_DELAYED}
  external OpenJpegDllName name S_OPJ_DECODER_SET_STRICT_MODE delayed;
{$ENDIF}

{**
 * Allocates worker threads for the compressor/decompressor.
 *
 * By default, only the main thread is used. If this function is not used,
 * but the OPJ_NUM_THREADS environment variable is set, its value will be
 * used to initialize the number of threads. The value can be either an integer
 * number, or "ALL_CPUS". If OPJ_NUM_THREADS is set and this function is called,
 * this function will override the behaviour of the environment variable.
 *
 * Note: currently only has effect on the decompressor.
 *
 * @param p_codec       decompressor handler
 * @param num_threads   number of threads.
 *
 * @return OPJ_TRUE     if the decoder is correctly set
 *}
function opj_codec_set_threads(p_codec:popj_codec_t; num_threads:nativeint):OPJ_BOOL; stdcall;
{$IFDEF OPENJP2_USE_DELAYED}
  external OpenJpegDllName name S_OPJ_CODEC_SET_THREADS delayed;
{$ENDIF}

{**
 * Decodes an image header.
 *
 * @param	p_stream		the jpeg2000 stream.
 * @param	p_codec			the jpeg2000 codec to read.
 * @param	p_image			the image structure initialized with the characteristics of encoded image.
 *
 * @return true				if the main header of the codestream and the JP2 header is correctly read.
 *}
function opj_read_header( p_stream: popj_stream_t; p_codec: popj_codec_t;
                          p_p_image: ppopj_image_t):OPJ_BOOL; stdcall;
{$IFDEF OPENJP2_USE_DELAYED}
  external OpenJpegDllName name S_OPJ_READ_HEADER delayed;
{$ENDIF}

{** Restrict the number of components to decode.
 *
 * This function should be called after opj_read_header().
 *
 * This function enables to restrict the set of decoded components to the
 * specified indices.
 * Note that the current implementation (apply_color_transforms == OPJ_FALSE)
 * is such that neither the multi-component transform at codestream level,
 * nor JP2 channel transformations will be applied.
 * Consequently the indices are relative to the codestream.
 *
 * Note: opj_decode_tile_data() should not be used together with opj_set_decoded_components().
 *
 * @param   p_codec         the jpeg2000 codec to read.
 * @param   numcomps        Size of the comps_indices array.
 * @param   comps_indices   Array of numcomps values representing the indices
 *                          of the components to decode (relative to the
 *                          codestream, starting at 0)
 * @param   apply_color_transforms Whether multi-component transform at codestream level
 *                                 or JP2 channel transformations should be applied.
 *                                 Currently this parameter should be set to OPJ_FALSE.
 *                                 Setting it to OPJ_TRUE will result in an error.
 *
 * @return OPJ_TRUE         in case of success.
 *}
function opj_set_decoded_components(p_codec: popj_codec_t;
                                    numcomps:OPJ_UINT32;
                                    pcomps_indices:POPJ_UINT32;
                                    apply_color_transforms:OPJ_BOOL):OPJ_BOOL; stdcall;
{$IFDEF OPENJP2_USE_DELAYED}
  external OpenJpegDllName name S_OPJ_SET_DECODED_COMPONENTS delayed;
{$ENDIF}

{**
 * Sets the given area to be decoded. This function should be called right after opj_read_header and before any tile header reading.
 *
 * The coordinates passed to this function should be expressed in the reference grid,
 * that is to say at the highest resolution level, even if requesting the image at lower
 * resolution levels.
 *
 * Generally opj_set_decode_area() should be followed by opj_decode(), and the
 * codec cannot be re-used.
 * In the particular case of an image made of a single tile, several sequences of
 * calls to opoj_set_decode_area() and opj_decode() are allowed, and will bring
 * performance improvements when reading an image by chunks.
 *
 * @param	p_codec			the jpeg2000 codec.
 * @param	p_image         the decoded image previously setted by opj_read_header
 * @param	p_start_x		the left position of the rectangle to decode (in image coordinates).
 * @param	p_end_x			the right position of the rectangle to decode (in image coordinates).
 * @param	p_start_y		the up position of the rectangle to decode (in image coordinates).
 * @param	p_end_y			the bottom position of the rectangle to decode (in image coordinates).
 *
 * @return	true			if the area could be set.
 *}
function opj_set_decode_area( p_codec: popj_codec_t; p_image: popj_image_t;
                              p_start_x, p_start_y, p_end_x, p_end_y: OPJ_INT32):OPJ_BOOL; stdcall;
{$IFDEF OPENJP2_USE_DELAYED}
  external OpenJpegDllName name S_OPJ_SET_DECODE_AREA delayed;
{$ENDIF}

{**
 * Decode an image from a JPEG-2000 codestream
 *
 * @param p_decompressor 	decompressor handle
 * @param p_stream			Input buffer stream
 * @param p_image 			the decoded image
 * @return 					true if success, otherwise false
 **}
function opj_decode( p_decompressor: popj_codec_t; p_stream: popj_stream_t;
                     p_image: popj_image_t):OPJ_BOOL; stdcall;
{$IFDEF OPENJP2_USE_DELAYED}
  external OpenJpegDllName name S_OPJ_DECODE delayed;
{$ENDIF}

{**
 * Get the decoded tile from the codec
 *
 * @param	p_codec			  the jpeg2000 codec.
 * @param	p_stream      input streamm
 * @param	p_image       output image
 * @param	tile_index		index of the tile which will be decode
 *
 * @return					true if success, otherwise false
 *}
function opj_get_decoded_tile( p_codec: popj_codec_t; p_stream: popj_stream_t;
                               p_image: popj_image_t; tile_index: OPJ_UINT32):OPJ_BOOL; stdcall;
{$IFDEF OPENJP2_USE_DELAYED}
  external OpenJpegDllName name S_OPJ_GET_DECODED_TILE delayed;
{$ENDIF}

{**
 * Set the resolution factor of the decoded image
 * @param	p_codec	    the jpeg2000 codec.
 * @param	res_factor  resolution factor to set
 *
 * @return					true if success, otherwise false
 *}
function opj_set_decoded_resolution_factor( p_codec: popj_codec_t; res_factor: OPJ_UINT32):OPJ_BOOL; stdcall;
{$IFDEF OPENJP2_USE_DELAYED}
  external OpenJpegDllName name S_OPJ_SET_DECODED_RESOLUTION_FACTOR delayed;
{$ENDIF}

{**
 * Writes a tile with the given data.
 *
 * @param	p_codec		    the jpeg2000 codec.
 * @param	p_tile_index	the index of the tile to write. At the moment, the tiles must be written from 0 to n-1 in sequence.
 * @param	p_data        pointer to the data to write. Data is arranged in sequence, data_comp0, then data_comp1, then ... NO INTERLEAVING should be set.
 * @param	p_data_size   this value os used to make sure the data being written is correct. The size must be equal to the sum for each component of
 *                      tile_width * tile_height * component_size. component_size can be 1,2 or 4 bytes, depending on the precision of the given component.
 * @param	p_stream	    the stream to write data to.
 *
 * @return	true if the data could be written.
 *}
function opj_write_tile( p_codec: popj_codec_t; p_tile_index: OPJ_UINT32; p_data: POPJ_BYTE;
                         p_data_size: OPJ_UINT32; p_stream: popj_stream_t):OPJ_BOOL; stdcall;
{$IFDEF OPENJP2_USE_DELAYED}
  external OpenJpegDllName name S_OPJ_WRITE_TILE delayed;
{$ENDIF}

{**
 * Reads a tile header. This function is compulsory and allows one to know the size of the tile thta will be decoded.
 * The user may need to refer to the image got by opj_read_header to understand the size being taken by the tile.
 *
 * @param	p_codec			the jpeg2000 codec.
 * @param	p_tile_index	pointer to a value that will hold the index of the tile being decoded, in case of success.
 * @param	p_data_size		pointer to a value that will hold the maximum size of the decoded data, in case of success. In case
 *							of truncated codestreams, the actual number of bytes decoded may be lower. The computation of the size is the same
 *							as depicted in opj_write_tile.
 * @param	p_tile_x0		pointer to a value that will hold the x0 pos of the tile (in the image).
 * @param	p_tile_y0		pointer to a value that will hold the y0 pos of the tile (in the image).
 * @param	p_tile_x1		pointer to a value that will hold the x1 pos of the tile (in the image).
 * @param	p_tile_y1		pointer to a value that will hold the y1 pos of the tile (in the image).
 * @param	p_nb_comps		pointer to a value that will hold the number of components in the tile.
 * @param	p_should_go_on	pointer to a boolean that will hold the fact that the decoding should go on. In case the
 *							codestream is over at the time of the call, the value will be set to false. The user should then stop
 *							the decoding.
 * @param	p_stream		the stream to decode.
 * @return	true			if the tile header could be decoded. In case the decoding should end, the returned value is still true.
 *							returning false may be the result of a shortage of memory or an internal error.
 *}
function opj_read_tile_header( p_codec: popj_codec_t; p_stream: popj_stream_t;
                               p_tile_index, p_data_size: POPJ_UINT32;
                               p_tile_x0, p_tile_y0, p_tile_x1, p_tile_y1: POPJ_INT32;
                               p_nb_comps: POPJ_UINT32; p_should_go_on: POPJ_BOOL):OPJ_BOOL; stdcall;
{$IFDEF OPENJP2_USE_DELAYED}
  external OpenJpegDllName name S_OPJ_READ_TILE_HEADER delayed;
{$ENDIF}

{**
 * Reads a tile data. This function is compulsory and allows one to decode tile data. opj_read_tile_header should be called before.
 * The user may need to refer to the image got by opj_read_header to understand the size being taken by the tile.
 *
 * Note: opj_decode_tile_data() should not be used together with opj_set_decoded_components().
 *
 * @param	p_codec			the jpeg2000 codec.
 * @param	p_tile_index	the index of the tile being decoded, this should be the value set by opj_read_tile_header.
 * @param	p_data			pointer to a memory block that will hold the decoded data.
 * @param	p_data_size		size of p_data. p_data_size should be bigger or equal to the value set by opj_read_tile_header.
 * @param	p_stream		the stream to decode.
 *
 * @return	true			if the data could be decoded.
 *}
function opj_decode_tile_data( p_codec: popj_codec_t; p_tile_index: OPJ_UINT32;
                               p_data: POPJ_BYTE; p_data_size: OPJ_UINT32;
                               p_stream: popj_stream_t):OPJ_BOOL; stdcall;
{$IFDEF OPENJP2_USE_DELAYED}
  external OpenJpegDllName name S_OPJ_DECODE_TILE_DATA delayed;
{$ENDIF}

{* COMPRESSION FUNCTIONS *}

{**
 * Creates a J2K/JP2 compression structure
 * @param 	format 		Coder to select
 * @return 				Returns a handle to a compressor if successful, returns NULL otherwise
 *}
function opj_create_compress( format: OPJ_CODEC_FORMAT ):popj_codec_t; stdcall;
{$IFDEF OPENJP2_USE_DELAYED}
  external OpenJpegDllName name S_OPJ_CREATE_COMPRESS delayed;
{$ENDIF}

{**
Set encoding parameters to default values, that means :
<ul>
<li>Lossless
<li>1 tile
<li>Size of precinct : 2^15 x 2^15 (means 1 precinct)
<li>Size of code-block : 64 x 64
<li>Number of resolutions: 6
<li>No SOP marker in the codestream
<li>No EPH marker in the codestream
<li>No sub-sampling in x or y direction
<li>No mode switch activated
<li>Progression order: LRCP
<li>No index file
<li>No ROI upshifted
<li>No offset of the origin of the image
<li>No offset of the origin of the tiles
<li>Reversible DWT 5-3
</ul>
@param parameters Compression parameters
*}
procedure opj_set_default_encoder_parameters( parameters: popj_cparameters_t ); stdcall;
{$IFDEF OPENJP2_USE_DELAYED}
  external OpenJpegDllName name S_OPJ_SET_DEFAULT_ENCODER_PARAMETERS delayed;
{$ENDIF}

{**
 * Setup the encoder parameters using the current image and using user parameters.
 * @param p_codec 		Compressor handle
 * @param parameters 	Compression parameters
 * @param image 		Input filled image
 *}
function opj_setup_encoder( p_codec: popj_codec_t; parameters: popj_cparameters_t;
                            p_image: popj_image_t): OPJ_BOOL; stdcall;
{$IFDEF OPENJP2_USE_DELAYED}
  external OpenJpegDllName name S_OPJ_SETUP_ENCODER delayed;
{$ENDIF}

{**
 * Specify extra options for the encoder.
 *
 * This may be called after opj_setup_encoder() and before opj_start_compress()
 *
 * This is the way to add new options in a fully ABI compatible way, without
 * extending the opj_cparameters_t structure.
 *
 * Currently supported options are:
 * <ul>
 * <li>PLT=YES/NO. Defaults to NO. If set to YES, PLT marker segments,
 *     indicating the length of each packet in the tile-part header, will be
 *     written. Since 2.3.2</li>
 * </ul>
 *
 * @param p_codec       Compressor handle
 * @param p_options     Compression options. This should be a NULL terminated
 *                      array of strings. Each string is of the form KEY=VALUE.
 *
 * @return OPJ_TRUE in case of success.
 * @since 2.3.2
 *}
type
  TOpjEncoderSetExtraOptions = array of PAnsiChar;
  POpjEncoderSetExtraOptions = ^TOpjEncoderSetExtraOptions;

function opj_encoder_set_extra_options( p_codec:popj_codec_t;
           p_options:POpjEncoderSetExtraOptions): OPJ_BOOL; stdcall;
{$IFDEF OPENJP2_USE_DELAYED}
  external OpenJpegDllName name S_OPJ_ENCODER_SET_EXTRA_OPTIONS delayed;
{$ENDIF}

{**
 * Start to compress the current image.
 * @param p_codec 		Compressor handle
 * @param image 	    Input filled image
 * @param p_stream 		Input stgream
 *}
function opj_start_compress( p_codec: popj_codec_t; p_image: popj_image_t;
                             p_stream: popj_stream_t): OPJ_BOOL; stdcall;
{$IFDEF OPENJP2_USE_DELAYED}
  external OpenJpegDllName name S_POJ_START_COMPRESS delayed;
{$ENDIF}

{**
 * End to compress the current image.
 * @param p_codec 		Compressor handle
 * @param p_stream 		Input stgream
 *}
function opj_end_compress( p_codec: popj_codec_t; p_stream: popj_stream_t): OPJ_BOOL; stdcall;
{$IFDEF OPENJP2_USE_DELAYED}
  external OpenJpegDllName name S_OPJ_END_COMPRESS delayed;
{$ENDIF}

{**
 * Encode an image into a JPEG-2000 codestream
 * @param p_codec 		compressor handle
 * @param p_stream 		Output buffer stream
 *
 * @return 				Returns true if successful, returns false otherwise
 *}
function opj_encode( p_codec:popj_codec_t; p_stream:popj_stream_t): OPJ_BOOL; stdcall;
{$IFDEF OPENJP2_USE_DELAYED}
  external OpenJpegDllName name S_OPJ_ENCODE delayed;
{$ENDIF}

{*
==========================================================
   MCT functions
==========================================================
*}

{**
 * Sets the MCT matrix to use.
 *
 * @param	parameters		the parameters to change.
 * @param	pEncodingMatrix	the encoding matrix.
 * @param	p_dc_shift		the dc shift coefficients to use.
 * @param	pNbComp			the number of components of the image.
 *
 * @return	true if the parameters could be set.
 *}
function opj_set_MCT( parameters: popj_cparameters_t;
                      pEncodingMatrix: POPJ_FLOAT32;
                      p_dc_shift: POPJ_INT32;
                      pNbComp: OPJ_UINT32): OPJ_BOOL; stdcall;
{$IFDEF OPENJP2_USE_DELAYED}
  external OpenJpegDllName name S_OPJ_SET_MCT delayed;
{$ENDIF}

{*
==========================================================
   Thread functions
==========================================================
*}

{** Returns if the library is built with thread support.
 * OPJ_TRUE if mutex, condition, thread, thread pool are available.
 *}
function opj_has_thread_support:OPJ_BOOL; stdcall;
{$IFDEF OPENJP2_USE_DELAYED}
  external OpenJpegDllName name S_OPJ_HAS_THREAD_SUPPORT delayed;
{$ENDIF}

{** Return the number of virtual CPUs *}
function opj_get_num_cpus:nativeint; stdcall;
{$IFDEF OPENJP2_USE_DELAYED}
  external OpenJpegDllName name S_OPJ_GET_NUM_CPUS delayed;
{$ENDIF}


// check to be enabled jpeg2000.dll
function EnabledJpeg2K:boolean;

{$IFNDEF OPENJP2_USE_DELAYED}
function  LoadOpenJp2:boolean;
procedure UnloadOpenJp2;
{$ENDIF}

implementation

{$IFDEF OPENJP2_USE_DELAYED}
{***************************************************************************
 *                                                                         *
 *                   for using with Delayed loading                        *
 *                                                                         *
 ***************************************************************************}

function EnabledJpeg2K:boolean;
begin
  result:=false;
  try
    result:=(nil<>opj_version); // need to check version ?
  except
    on E:Exception do ;
  end;
end;

{$ELSE}
{***************************************************************************
 *                                                                         *
 *               for using with LoadLibrary/FreeLibrary                    *
 *                                                                         *
 ***************************************************************************}

var
  hOpenjp2 : THandle;

  _opj_version:function:PAnsiChar; stdcall;

  _opj_image_create:function(numcmpts: OPJ_UINT32; cmptparms: popj_image_cmptparm_t;
                             clrspc: opj_COLOR_SPACE): popj_image_t; stdcall;
  _opj_image_destroy:procedure(image: popj_image_t); stdcall;
  _opj_image_tile_create:function(numcmpts: OPJ_UINT32; cmptparms: popj_image_cmptparm_t;
                                  clrspc: OPJ_COLOR_SPACE):popj_image_t; stdcall;
  _opj_image_data_alloc:function(size:OPJ_SIZE_T):pointer; stdcall;
  _opj_image_data_free:procedure(ptr:pointer); stdcall;

  _opj_stream_default_create:function(p_is_input:OPJ_BOOL):popj_stream_t ; stdcall;
  _opj_stream_create:function(p_buffer_size:OPJ_SIZE_T; p_is_input:OPJ_BOOL):popj_stream_t; stdcall;
  _opj_stream_destroy:procedure(p_stream:popj_stream_t); stdcall;
  _opj_stream_set_read_function:procedure(p_stream: popj_stream_t; p_function: opj_stream_read_fn); stdcall;
  _opj_stream_set_write_function:procedure(p_stream: popj_stream_t; p_function: opj_stream_write_fn); stdcall;
  _opj_stream_set_skip_function:procedure(p_stream: popj_stream_t; p_function: opj_stream_skip_fn); stdcall;
  _opj_stream_set_seek_function:procedure(p_stream: popj_stream_t; p_function: opj_stream_seek_fn); stdcall;
  _opj_stream_set_user_data:procedure(p_stream: popj_stream_t; p_data: pointer;
                                      p_function: opj_stream_free_user_data_fn); stdcall;
  _opj_stream_set_user_data_length:procedure(p_stream: popj_stream_t;
                                             data_length: OPJ_UINT64); stdcall;
  _opj_stream_create_default_file_stream:function(fname:PAnsiChar;
                                                  p_is_read_stream:OPJ_BOOL):popj_stream_t; stdcall;
  _opj_stream_create_file_stream:function( fname: PAnsiChar; p_buffer_size: OPJ_SIZE_T;
                                           p_is_read_stream: OPJ_BOOL):popj_stream_t; stdcall;

  _opj_set_info_handler:function( p_codec: popj_codec_t; p_callback: topj_msg_callback;
                                  p_user_data: pointer):OPJ_BOOL; stdcall;
  _opj_set_warning_handler:function( p_codec: popj_codec_t; p_callback: topj_msg_callback;
                                     p_user_data: pointer):OPJ_BOOL; stdcall;
  _opj_set_error_handler:function( p_codec: popj_codec_t; p_callback: topj_msg_callback;
                                   p_user_data: pointer):OPJ_BOOL; stdcall;

  _opj_create_decompress:function(format: OPJ_CODEC_FORMAT):POPJ_CODEC_T; stdcall;
  _opj_destroy_codec:procedure(p_codec: popj_codec_t); stdcall;
  _opj_end_decompress:function( p_codec: popj_codec_t; p_stream: popj_stream_t):OPJ_BOOL; stdcall;
  _opj_set_default_decoder_parameters:procedure( parameters: popj_dparameters_t); stdcall;
  _opj_setup_decoder:function( p_codec: popj_codec_t; parameters: popj_dparameters_t):OPJ_BOOL; stdcall;
  _opj_decoder_set_strict_mode:function( p_codec: popj_codec_t; strict:OPJ_BOOL):OPJ_BOOL; stdcall;
  _opj_codec_set_threads:function(p_codec:popj_codec_t; num_threads:nativeint):OPJ_BOOL; stdcall;
  _opj_read_header:function( p_stream: popj_stream_t; p_codec: popj_codec_t;
                             p_p_image: ppopj_image_t):OPJ_BOOL; stdcall;
  _opj_set_decoded_components:function(p_codec:popj_codec_t;
                                       numcomps:OPJ_UINT32;
                                       pcomps_indices:POPJ_UINT32;
                                       apply_color_transforms:OPJ_BOOL):OPJ_BOOL; stdcall;
  _opj_set_decode_area:function( p_codec: popj_codec_t; p_image: popj_image_t;
                                 p_start_x, p_start_t, p_end_x, p_end_y: OPJ_INT32):OPJ_BOOL; stdcall;
  _opj_decode:function( p_decompressor: popj_codec_t; p_stream: popj_stream_t;
                        p_image: popj_image_t):OPJ_BOOL; stdcall;
  _opj_get_decoded_tile:function( p_codec: popj_codec_t; p_stream: popj_stream_t;
                                  p_image: popj_image_t; tile_index: OPJ_UINT32):OPJ_BOOL; stdcall;
  _opj_set_decoded_resolution_factor:function( p_codec: popj_codec_t;
                                               res_factor: OPJ_UINT32):OPJ_BOOL; stdcall;
  _opj_write_tile:function( p_codec: popj_codec_t; p_tile_index: OPJ_UINT32; p_data: POPJ_BYTE;
                            p_data_size: OPJ_UINT32; p_stream: popj_stream_t):OPJ_BOOL; stdcall;
  _opj_read_tile_header:function( p_codec: popj_codec_t; p_stream: popj_stream_t;
                                  p_tile_index, p_data_size: POPJ_UINT32;
                                  p_tile_x0, p_tile_y0, p_tile_x1, p_tile_u1: POPJ_INT32;
                                  p_nb_comps: POPJ_UINT32; p_should_go_on: POPJ_BOOL):OPJ_BOOL; stdcall;
  _opj_decode_tile_data:function( p_codec: popj_codec_t; p_tile_index: OPJ_UINT32;
                                  p_data: POPJ_BYTE; p_data_size: OPJ_UINT32;
                                  p_stream: popj_stream_t):OPJ_BOOL; stdcall;
  _opj_create_compress:function( format: OPJ_CODEC_FORMAT ):popj_codec_t; stdcall;
  _opj_set_default_encoder_parameters:procedure( parameters: popj_cparameters_t ); stdcall;
  _opj_setup_encoder:function( p_codec: popj_codec_t; parameters: popj_cparameters_t;
                               p_image: popj_image_t): OPJ_BOOL; stdcall;
  _opj_start_compress:function( p_codec: popj_codec_t; p_image: popj_image_t;
                                p_stream: popj_stream_t): OPJ_BOOL; stdcall;
  _opj_end_compress:function( p_codec: popj_codec_t; p_stream: popj_stream_t): OPJ_BOOL; stdcall;
  _opj_encode:function( p_codec:popj_codec_t; p_stream:popj_stream_t): OPJ_BOOL; stdcall;
  _opj_encoder_set_extra_options:function( p_codec:popj_codec_t;
                                           p_options:POpjEncoderSetExtraOptions): OPJ_BOOL; stdcall;

  _opj_set_MCT:function( parameters: popj_cparameters_t;
                         pEncodingMatrix: POPJ_FLOAT32;
                         p_dc_shift: POPJ_INT32;
                         pNbComp: OPJ_UINT32): OPJ_BOOL; stdcall;

  _opj_has_thread_support:function:OPJ_BOOL; stdcall;
  _opj_get_num_cpus:function:nativeint; stdcall;

function e_opj_version: PAnsiChar; stdcall;
begin
  raise EOpenJp2Error.Create('cannot use opj_version');
end;

function opj_version: PAnsiChar; stdcall;
begin
  result:=_opj_version;
end;

function e_opj_image_create(numcmpts: OPJ_UINT32; cmptparms: popj_image_cmptparm_t;
  clrspc: opj_COLOR_SPACE): popj_image_t; stdcall;
begin
  raise EOpenJp2Error.Create('cannnot use opj_image_create');
end;

function opj_image_create(numcmpts: OPJ_UINT32; cmptparms: popj_image_cmptparm_t;
  clrspc: opj_COLOR_SPACE): popj_image_t; stdcall;
begin
  result:=_opj_image_create(numcmpts,cmptparms,clrspc);
end;

procedure e_opj_image_destroy(image: popj_image_t); stdcall;
begin
  raise EOpenJp2Error.Create('cannnot use opj_image_destroy');
end;

procedure opj_image_destroy(image: popj_image_t); stdcall;
begin
  _opj_image_destroy(image);
end;

function e_opj_image_tile_create(numcmpts: OPJ_UINT32; cmptparms: popj_image_cmptparm_t; clrspc: OPJ_COLOR_SPACE):popj_image_t;
  stdcall;
begin
  raise EOpenJp2Error.Create('cannot use opj_image_tile_create');
end;

function opj_image_tile_create(numcmpts: OPJ_UINT32; cmptparms: popj_image_cmptparm_t; clrspc: OPJ_COLOR_SPACE):popj_image_t;
  stdcall;
begin
  result:=_opj_image_tile_create(numcmpts,cmptparms,clrspc);
end;

function e_opj_image_data_alloc(size:OPJ_SIZE_T):pointer; stdcall;
begin
  raise EOpenJp2Error.Create('cannot use opj_image_data_alloc');
end;

function opj_image_data_alloc(size:OPJ_SIZE_T):pointer; stdcall;
begin
  result:=_opj_image_data_alloc(size);
end;

procedure e_opj_image_data_free(ptr:pointer); stdcall;
begin
  raise EOpenJp2Error.Create('cannot use opj_image_data_free');
end;

procedure opj_image_data_free(ptr:pointer); stdcall;
begin
  _opj_image_data_free(ptr);
end;

function e_opj_stream_default_create(p_is_input:OPJ_BOOL):popj_stream_t ; stdcall;
begin
  raise EOpenJp2Error.Create('cannot use opj_stream_default_create');
end;

function opj_stream_default_create(p_is_input:OPJ_BOOL):popj_stream_t ; stdcall;
begin
  result:=_opj_stream_default_create(p_is_input);
end;

function e_opj_stream_create(p_buffer_size:OPJ_SIZE_T; p_is_input:OPJ_BOOL):popj_stream_t; stdcall;
begin
  raise EOpenJp2Error.Create('cannot use opj_stream_create');
end;

function opj_stream_create(p_buffer_size:OPJ_SIZE_T; p_is_input:OPJ_BOOL):popj_stream_t; stdcall;
begin
  result:=_opj_stream_create(p_buffer_size,p_is_input);
end;

procedure e_opj_stream_destroy(p_stream:popj_stream_t); stdcall;
begin
  raise EOpenJp2Error.Create('cannot use opj_stream_destroy');
end;

procedure opj_stream_destroy(p_stream:popj_stream_t); stdcall;
begin
  _opj_stream_destroy(p_stream);
end;

procedure e_opj_stream_set_read_function(p_stream: popj_stream_t;
                                         p_function: opj_stream_read_fn); stdcall;
begin
  raise EOpenJp2Error.Create('cannot use opj_stream_set_read_function');
end;

procedure opj_stream_set_read_function(p_stream: popj_stream_t;
                                       p_function: opj_stream_read_fn); stdcall;
begin
  _opj_stream_set_read_function(p_stream,p_function);
end;

procedure e_opj_stream_set_write_function(p_stream: popj_stream_t;
                                          p_function: opj_stream_write_fn); stdcall;
begin
  raise EOpenJp2Error.Create('cannot use opj_stream_set_write_function');
end;

procedure opj_stream_set_write_function(p_stream: popj_stream_t;
                                        p_function: opj_stream_write_fn); stdcall;
begin
  _opj_stream_set_write_function(p_stream,p_function);
end;

procedure e_opj_stream_set_skip_function(p_stream: popj_stream_t;
                                         p_function: opj_stream_skip_fn); stdcall;
begin
  raise EOpenJp2Error.Create('cannot use opj_stream_set_skip_function');
end;

procedure opj_stream_set_skip_function(p_stream: popj_stream_t;
                                       p_function: opj_stream_skip_fn); stdcall;
begin
  _opj_stream_set_skip_function(p_stream,p_function);
end;

procedure e_opj_stream_set_seek_function(p_stream: popj_stream_t;
                                         p_function: opj_stream_seek_fn); stdcall;
begin
  raise EOpenJp2Error.Create('cannot use opj_stream_set_seek_function');
end;

procedure opj_stream_set_seek_function(p_stream: popj_stream_t;
                                       p_function: opj_stream_seek_fn); stdcall;
begin
  _opj_stream_set_seek_function(p_stream,p_function);
end;

procedure e_opj_stream_set_user_data(p_stream: popj_stream_t; p_data: pointer;
                                     p_function: opj_stream_free_user_data_fn); stdcall;
begin
  raise EOpenJp2Error.Create('cannot use opj_stream_set_user_data');
end;

procedure opj_stream_set_user_data(p_stream: popj_stream_t; p_data: pointer;
                                   p_function: opj_stream_free_user_data_fn); stdcall;
begin
  _opj_stream_set_user_data(p_stream,p_data,p_function);
end;

procedure e_opj_stream_set_user_data_length(p_stream: popj_stream_t;
                                            data_length: OPJ_UINT64); stdcall;
begin
  raise EOpenJp2Error.Create('cannot use opj_stream_set_user_data_length');
end;

procedure opj_stream_set_user_data_length(p_stream: popj_stream_t;
                                          data_length: OPJ_UINT64); stdcall;
begin
  _opj_stream_set_user_data_length(p_stream,data_length);
end;

function e_opj_stream_create_default_file_stream( fname : PAnsiChar;
                                                  p_is_read_stream: OPJ_BOOL):popj_stream_t; stdcall;
begin
  raise EOpenJp2Error.Create('cannot use opj_stream_create_default_file_stream');
end;

function opj_stream_create_default_file_stream( fname : PAnsiChar;
                                                p_is_read_stream: OPJ_BOOL):popj_stream_t; stdcall;
begin
  result:=_opj_stream_create_default_file_stream(fname,p_is_read_stream);
end;

function e_opj_stream_create_file_stream( fname: PAnsiChar; p_buffer_size: OPJ_SIZE_T;
                                          p_is_read_stream: OPJ_BOOL):popj_stream_t; stdcall;
begin
  raise EOpenJp2Error.Create('cannot use opj_stream_create_file_stream');
end;

function opj_stream_create_file_stream( fname: PAnsiChar; p_buffer_size: OPJ_SIZE_T;
                                        p_is_read_stream: OPJ_BOOL):popj_stream_t; stdcall;
begin
  result:=_opj_stream_create_file_stream(fname,p_buffer_size,p_is_read_stream);
end;

function e_opj_set_info_handler( p_codec: popj_codec_t; p_callback: topj_msg_callback;
                                 p_user_data: pointer):OPJ_BOOL; stdcall;
begin
  raise EOpenJp2Error.Create('cannot use opj_set_info_handler');
end;

function opj_set_info_handler( p_codec: popj_codec_t; p_callback: topj_msg_callback;
                               p_user_data: pointer):OPJ_BOOL; stdcall;
begin
  result:=_opj_set_info_handler(p_codec,p_callback,p_user_data);
end;

function e_opj_set_warning_handler( p_codec: popj_codec_t; p_callback: topj_msg_callback;
                                    p_user_data: pointer):OPJ_BOOL; stdcall;
begin
  raise EOpenJp2Error.Create('cannot use opj_set_warning_handler');
end;

function opj_set_warning_handler( p_codec: popj_codec_t; p_callback: topj_msg_callback;
                                  p_user_data: pointer):OPJ_BOOL; stdcall;
begin
  result:=_opj_set_warning_handler(p_codec,p_callback,p_user_data);
end;

function e_opj_set_error_handler( p_codec: popj_codec_t; p_callback: topj_msg_callback;
                                  p_user_data: pointer):OPJ_BOOL; stdcall;
begin
  raise EOpenJp2Error.Create('cannot use opj_set_error_handler');
end;

function opj_set_error_handler( p_codec: popj_codec_t; p_callback: topj_msg_callback;
                                p_user_data: pointer):OPJ_BOOL; stdcall;
begin
  result:=_opj_set_error_handler(p_codec,p_callback,p_user_data);
end;

function e_opj_create_decompress(format: OPJ_CODEC_FORMAT):POPJ_CODEC_T; stdcall;
begin
  raise EOpenJp2Error.Create('cannot use opj_create_decompress');
end;

function opj_create_decompress(format: OPJ_CODEC_FORMAT):POPJ_CODEC_T; stdcall;
begin
  result:=_opj_create_decompress(format);
end;

procedure e_opj_destroy_codec(p_codec: popj_codec_t); stdcall;
begin
  raise EOpenJp2Error.Create('cannot use opj_destroy_codec');
end;

procedure opj_destroy_codec(p_codec: popj_codec_t); stdcall;
begin
  _opj_destroy_codec(p_codec);
end;

function e_opj_end_decompress( p_codec: popj_codec_t;
                               p_stream: popj_stream_t):OPJ_BOOL; stdcall;
begin
  raise EOpenJp2Error.Create('cannot use opj_end_decompress');
end;

function opj_end_decompress( p_codec: popj_codec_t;
                             p_stream: popj_stream_t):OPJ_BOOL; stdcall;
begin
  result:=_opj_end_decompress(p_codec,p_stream);
end;

procedure e_opj_set_default_decoder_parameters( parameters: popj_dparameters_t); stdcall;
begin
  raise EOpenJp2Error.Create('cannot use opj_set_default_decoder_parameters');
end;

procedure opj_set_default_decoder_parameters( parameters: popj_dparameters_t); stdcall;
begin
  _opj_set_default_decoder_parameters( parameters );
end;

function e_opj_setup_decoder( p_codec: popj_codec_t; parameters: popj_dparameters_t):OPJ_BOOL; stdcall;
begin
  raise EOpenJp2Error.Create('cannot use opj_setup_decoder');
end;

function opj_setup_decoder( p_codec: popj_codec_t; parameters: popj_dparameters_t):OPJ_BOOL; stdcall;
begin
  result:=_opj_setup_decoder( p_codec, parameters );
end;

function e_opj_decoder_set_strict_mode( p_codec: popj_codec_t; strict:OPJ_BOOL):OPJ_BOOL; stdcall;
begin
  raise EOpenJp2Error.Create('cannot use opj_decoder_set_strict_mode');
end;

function opj_decoder_set_strict_mode( p_codec: popj_codec_t; strict:OPJ_BOOL):OPJ_BOOL; stdcall;
begin
  result:=_opj_decoder_set_strict_mode( p_codec, strict );
end;

function e_opj_codec_set_threads(p_codec:popj_codec_t; num_threads:nativeint):OPJ_BOOL; stdcall;
begin
  raise EOpenJp2Error.Create('cannot use opj_codec_set_threads');
end;

function opj_codec_set_threads(p_codec:popj_codec_t; num_threads:nativeint):OPJ_BOOL; stdcall;
begin
  result:=_opj_codec_set_threads(p_codec,num_threads);
end;

function e_opj_read_header( p_stream: popj_stream_t; p_codec: popj_codec_t;
                            p_p_image: ppopj_image_t):OPJ_BOOL; stdcall;
begin
  raise EOpenJp2Error.Create('cannot use opj_read_header');
end;

function opj_read_header( p_stream: popj_stream_t; p_codec: popj_codec_t;
                          p_p_image: ppopj_image_t):OPJ_BOOL; stdcall;
begin
  result:=_opj_read_header(p_stream,p_codec,p_p_image);
end;

function e_opj_set_decoded_components(p_codec: popj_codec_t;
                                      numcomps:OPJ_UINT32;
                                      pcomps_indices:POPJ_UINT32;
                                      apply_color_transforms:OPJ_BOOL):OPJ_BOOL; stdcall;
begin
  raise EOpenJp2Error.Create('cannot use opj_set_decoded_components');
end;

function opj_set_decoded_components(p_codec: popj_codec_t;
                                      numcomps:OPJ_UINT32;
                                      pcomps_indices:POPJ_UINT32;
                                      apply_color_transforms:OPJ_BOOL):OPJ_BOOL; stdcall;
begin
  result:=_opj_set_decoded_components(p_codec,numcomps,pcomps_indices,apply_color_transforms);
end;

function e_opj_set_decode_area( p_codec: popj_codec_t; p_image: popj_image_t;
                                p_start_x, p_start_y, p_end_x, p_end_y: OPJ_INT32):OPJ_BOOL; stdcall;
begin
  raise EOpenJp2Error.Create('cannot use opj_set_decode_area');
end;

function opj_set_decode_area( p_codec: popj_codec_t; p_image: popj_image_t;
                              p_start_x, p_start_y, p_end_x, p_end_y: OPJ_INT32):OPJ_BOOL; stdcall;
begin
  result:=_opj_set_decode_area(p_codec,p_image,p_start_x,p_start_y,p_end_x,p_end_y);
end;

function e_opj_decode( p_decompressor: popj_codec_t; p_stream: popj_stream_t;
                       p_image: popj_image_t):OPJ_BOOL; stdcall;
begin
  raise EOpenJp2Error.Create('cannot use opj_decode');
end;

function opj_decode( p_decompressor: popj_codec_t; p_stream: popj_stream_t;
                     p_image: popj_image_t):OPJ_BOOL; stdcall;
begin
  result:=_opj_decode(p_decompressor,p_stream,p_image);
end;

function e_opj_get_decoded_tile( p_codec: popj_codec_t; p_stream: popj_stream_t;
                                 p_image: popj_image_t; tile_index: OPJ_UINT32):OPJ_BOOL; stdcall;
begin
  raise EOpenJp2Error.Create('cannot use opj_get_decoded_tile');
end;

function opj_get_decoded_tile( p_codec: popj_codec_t; p_stream: popj_stream_t;
                               p_image: popj_image_t; tile_index: OPJ_UINT32):OPJ_BOOL; stdcall;
begin
  result:=_opj_get_decoded_tile(p_codec,p_stream,p_image,tile_index);
end;

function e_opj_set_decoded_resolution_factor( p_codec: popj_codec_t;
                                              res_factor: OPJ_UINT32):OPJ_BOOL; stdcall;
begin
  raise EOpenJp2Error.Create('cannot use opj_set_decoded_resolution_factor');
end;

function opj_set_decoded_resolution_factor( p_codec: popj_codec_t;
                                            res_factor: OPJ_UINT32):OPJ_BOOL; stdcall;
begin
  result:=_opj_set_decoded_resolution_factor(p_codec,res_factor);
end;

function e_opj_write_tile( p_codec: popj_codec_t; p_tile_index: OPJ_UINT32; p_data: POPJ_BYTE;
                           p_data_size: OPJ_UINT32; p_stream: popj_stream_t):OPJ_BOOL; stdcall;
begin
  raise EOpenJp2Error.Create('cannot use opj_write_tile');
end;

function opj_write_tile( p_codec: popj_codec_t; p_tile_index: OPJ_UINT32; p_data: POPJ_BYTE;
                         p_data_size: OPJ_UINT32; p_stream: popj_stream_t):OPJ_BOOL; stdcall;
begin
  result:=_opj_write_tile(p_codec,p_tile_index,p_data,p_data_size,p_stream);
end;

function e_opj_read_tile_header( p_codec: popj_codec_t; p_stream: popj_stream_t;
                               p_tile_index, p_data_size: POPJ_UINT32;
                               p_tile_x0, p_tile_y0, p_tile_x1, p_tile_y1: POPJ_INT32;
                               p_nb_comps: POPJ_UINT32; p_should_go_on: POPJ_BOOL):OPJ_BOOL; stdcall;
begin
  raise EOpenJp2Error.Create('cannot use opj_read_tile_header');
end;

function opj_read_tile_header( p_codec: popj_codec_t; p_stream: popj_stream_t;
                               p_tile_index, p_data_size: POPJ_UINT32;
                               p_tile_x0, p_tile_y0, p_tile_x1, p_tile_y1: POPJ_INT32;
                               p_nb_comps: POPJ_UINT32; p_should_go_on: POPJ_BOOL):OPJ_BOOL; stdcall;
begin
  result:=_opj_read_tile_header(p_codec,p_stream,p_tile_index,p_data_size,
                                p_tile_x0, p_tile_y0, p_tile_x1, p_tile_y1,
                                p_nb_comps,p_should_go_on);
end;

function e_opj_decode_tile_data( p_codec: popj_codec_t; p_tile_index: OPJ_UINT32;
                                 p_data: POPJ_BYTE; p_data_size: OPJ_UINT32;
                                 p_stream: popj_stream_t):OPJ_BOOL; stdcall;
begin
  raise EOpenJp2Error.Create('cannot use opj_decode_tile_data');
end;

function opj_decode_tile_data( p_codec: popj_codec_t; p_tile_index: OPJ_UINT32;
                               p_data: POPJ_BYTE; p_data_size: OPJ_UINT32;
                               p_stream: popj_stream_t):OPJ_BOOL; stdcall;
begin
  result:=_opj_decode_tile_data(p_codec,p_tile_index,p_data,p_data_size,p_stream);
end;

function e_opj_create_compress( format: OPJ_CODEC_FORMAT ):popj_codec_t; stdcall;
begin
  raise EOpenJp2Error.Create('cannot use opj_create_compress');
end;

function opj_create_compress( format: OPJ_CODEC_FORMAT ):popj_codec_t; stdcall;
begin
  result:=_opj_create_compress( format );
end;

procedure e_opj_set_default_encoder_parameters( parameters: popj_cparameters_t ); stdcall;
begin
  raise EOpenJp2Error.Create('cannot use opj_set_default_encoder_parameters');
end;

procedure opj_set_default_encoder_parameters( parameters: popj_cparameters_t ); stdcall;
begin
  _opj_set_default_encoder_parameters( parameters );
end;

function e_opj_setup_encoder( p_codec: popj_codec_t; parameters: popj_cparameters_t;
                              p_image: popj_image_t): OPJ_BOOL; stdcall;
begin
  raise EOpenJp2Error.Create('cannot use opj_setup_encoder');
end;

function opj_setup_encoder( p_codec: popj_codec_t; parameters: popj_cparameters_t;
                            p_image: popj_image_t): OPJ_BOOL; stdcall;
begin
  result:=_opj_setup_encoder( p_codec, parameters, p_image );
end;

function e_opj_start_compress( p_codec: popj_codec_t; p_image: popj_image_t;
                               p_stream: popj_stream_t): OPJ_BOOL; stdcall;
begin
  raise EOpenJp2Error.Create('cannot use opj_start_compress');
end;

function opj_start_compress( p_codec: popj_codec_t; p_image: popj_image_t;
                             p_stream: popj_stream_t): OPJ_BOOL; stdcall;
begin
  result:=_opj_start_compress( p_codec, p_image, p_stream );
end;

function e_opj_end_compress( p_codec: popj_codec_t; p_stream: popj_stream_t): OPJ_BOOL; stdcall;
begin
  raise EOpenJp2Error.Create('cannot use opj_end_compress');
end;

function opj_end_compress( p_codec: popj_codec_t; p_stream: popj_stream_t): OPJ_BOOL; stdcall;
begin
  result:=_opj_end_compress( p_codec, p_stream );
end;

function e_opj_encode( p_codec:popj_codec_t; p_stream:popj_stream_t): OPJ_BOOL; stdcall;
begin
  raise EOpenJp2Error.Create('cannot use opj_encode');
end;

function opj_encode( p_codec:popj_codec_t; p_stream:popj_stream_t): OPJ_BOOL; stdcall;
begin
  result:=_opj_encode( p_codec, p_stream );
end;

function e_opj_encoder_set_extra_options( p_codec:popj_codec_t;
                                           p_options:POpjEncoderSetExtraOptions): OPJ_BOOL; stdcall;
begin
  raise EOpenJp2Error.Create('cannot use opj_encoder_set_extra_options');
end;

function opj_encoder_set_extra_options( p_codec:popj_codec_t;
                                           p_options:POpjEncoderSetExtraOptions): OPJ_BOOL; stdcall;
begin
  result:=_opj_encoder_set_extra_options(p_codec,p_options);
end;

function e_opj_set_MCT( parameters: popj_cparameters_t;
                        pEncodingMatrix: POPJ_FLOAT32;
                        p_dc_shift: POPJ_INT32;
                        pNbComp: OPJ_UINT32): OPJ_BOOL; stdcall;
begin
  raise EOpenJp2Error.Create('cannot use opj_set_MCT');
end;

function opj_set_MCT( parameters: popj_cparameters_t;
                      pEncodingMatrix: POPJ_FLOAT32;
                      p_dc_shift: POPJ_INT32;
                      pNbComp: OPJ_UINT32): OPJ_BOOL; stdcall;
begin
  result:=_opj_set_MCT( parameters, pEncodingMatrix, p_dc_shift, pNbComp );
end;

function e_opj_has_thread_support:OPJ_BOOL; stdcall;
begin
  raise EOpenJp2Error.Create('cannot use opj_has_thread_support');
end;

function opj_has_thread_support:OPJ_BOOL; stdcall;
begin
  result:=_opj_has_thread_support;
end;

function e_opj_get_num_cpus:nativeint; stdcall;
begin
  raise EOpenJp2Error.Create('cannot use opj_get_num_cpus');
end;

function opj_get_num_cpus:nativeint; stdcall;
begin
  result:=_opj_get_num_cpus;
end;

function EnabledJpeg2K:boolean;
begin
  try
    result:=(hOpenjp2<>0);
  except
    ON EOpenJp2Error DO
      result:=false;
  end;
end;

procedure Init;
begin
  hOpenjp2:=0;

  _opj_version:=e_opj_version;

  _opj_image_create      := e_opj_image_create;
  _opj_image_destroy     := e_opj_image_destroy;
  _opj_image_tile_create := e_opj_image_tile_create;

  _opj_image_data_alloc  := e_opj_image_data_alloc;
  _opj_image_data_free   := e_opj_image_data_free;

  _opj_stream_default_create             := e_opj_stream_default_create;
  _opj_stream_create                     := e_opj_stream_create;
  _opj_stream_destroy                    := e_opj_stream_destroy;
  _opj_stream_set_read_function          := e_opj_stream_set_read_function;
  _opj_stream_set_write_function         := e_opj_stream_set_write_function;
  _opj_stream_set_skip_function          := e_opj_stream_set_skip_function;
  _opj_stream_set_seek_function          := e_opj_stream_set_seek_function;
  _opj_stream_set_user_data              := e_opj_stream_set_user_data;
  _opj_stream_set_user_data_length       := e_opj_stream_set_user_data_length;
  _opj_stream_create_default_file_stream := e_opj_stream_create_default_file_stream;
  _opj_stream_create_file_stream         := e_opj_stream_create_file_stream;

  _opj_set_info_handler    := e_opj_set_info_handler;
  _opj_set_warning_handler := e_opj_set_warning_handler;
  _opj_set_error_handler   := e_opj_set_error_handler;

  _opj_create_decompress:=e_opj_create_decompress;
  _opj_destroy_codec:=e_opj_destroy_codec;
  _opj_end_decompress:=e_opj_end_decompress;
  _opj_set_default_decoder_parameters:=e_opj_set_default_decoder_parameters;
  _opj_setup_decoder:=e_opj_setup_decoder;
  _opj_decoder_set_strict_mode:=e_opj_decoder_set_strict_mode;
  _opj_codec_set_threads:=e_opj_codec_set_threads;
  _opj_read_header:=e_opj_read_header;
  _opj_set_decoded_components:=e_opj_set_decoded_components;
  _opj_set_decode_area:=e_opj_set_decode_area;
  _opj_decode:=e_opj_decode;
  _opj_get_decoded_tile:=e_opj_get_decoded_tile;
  _opj_set_decoded_resolution_factor:=e_opj_set_decoded_resolution_factor;
  _opj_write_tile:=e_opj_write_tile;
  _opj_read_tile_header:=e_opj_read_tile_header;
  _opj_decode_tile_data:=e_opj_decode_tile_data;
  _opj_create_compress:=e_opj_create_compress;
  _opj_set_default_encoder_parameters:=e_opj_set_default_encoder_parameters;
  _opj_setup_encoder:=e_opj_setup_encoder;
  _opj_start_compress:=e_opj_start_compress;
  _opj_end_compress:=e_opj_end_compress;
  _opj_encode:=e_opj_encode;
  _opj_encoder_set_extra_options:=e_opj_encoder_set_extra_options;

  _opj_set_MCT:=e_opj_set_MCT;

  _opj_has_thread_support:=e_opj_has_thread_support;
  _opj_get_num_cpus:=e_opj_get_num_cpus;
end;

function LoadOpenJp2:boolean;
var
  sv : cardinal;
  p : TFarProc;
begin
  result:=false;
  if hOpenJp2<>0 then begin
    result:=true; exit;
  end;
  //
  sv:=SetErrorMode(SEM_FAILCRITICALERRORS);
  try
    hOpenjp2:=LoadLibrary(OpenJpegDllName);
  finally
    SetErrorMode(sv);
  end;
  if hOpenjp2=0 then exit;
  //
  p:=GetProcAddress(hOpenJp2,S_OPJ_VERSION);
  if p<>nil then @_opj_version:=p;

  p:=GetProcAddress(hOpenJp2,S_OPJ_IMAGE_CREATE);
  if p<>nil then @_opj_image_create:=p;

  p:=GetProcAddress(hOpenJp2,S_OPJ_IMAGE_DESTROY);
  if p<>nil then @_opj_image_destroy:=p;

  p:=GetProcAddress(hOpenJp2,S_OPJ_IMAGE_TILE_CREATE);
  if p<>nil then @_opj_image_tile_create:=p;

  p:=GetProcAddress(hOpenJp2,S_OPJ_IMAGE_DATA_ALLOC);
  if p<>nil then @_opj_image_data_alloc:=p;

  p:=GetProcAddress(hOpenJp2,S_OPJ_IMAGE_DATA_FREE);
  if p<>nil then @_opj_image_data_free:=p;

  p:=GetProcAddress(hOpenJp2,S_OPJ_STREAM_DEFAULT_CREATE);
  if p<>nil then @_opj_stream_default_create:=p;

  p:=GetProcAddress(hOpenJp2,S_OPJ_STREAM_CREATE);
  if p<>nil then @_opj_stream_create:=p;

  p:=GetProcAddress(hOpenJp2,S_OPJ_STREAM_DESTROY);
  if p<>nil then @_opj_stream_destroy:=p;

  p:=GetProcAddress(hOpenJp2,S_OPJ_STREAM_SET_READ_FUNCTION);
  if p<>nil then @_opj_stream_set_read_function:=p;

  p:=GetProcAddress(hOpenJp2,S_OPJ_STREAM_SET_WRITE_FUNCTION);
  if p<>nil then @_opj_stream_set_write_function:=p;

  p:=GetProcAddress(hOpenJp2,S_OPJ_STREAM_SET_SKIP_FUNCTION);
  if p<>nil then @_opj_stream_set_skip_function:=p;

  p:=GetProcAddress(hOpenJp2,S_OPJ_STREAM_SET_SEEK_FUNCTION);
  if p<>nil then @_opj_stream_set_seek_function:=p;

  p:=GetProcAddress(hOpenJp2,S_OPJ_STREAM_SET_USER_DATA);
  if p<>nil then @_opj_stream_set_user_data:=p;

  p:=GetProcAddress(hOpenJp2,S_OPJ_STREAM_SET_USER_DATA_LENGTH);
  if p<>nil then @_opj_stream_set_user_data_length:=p;

  p:=GetProcAddress(hOpenJp2,S_OPJ_STREAM_CREATE_DEFAULT_FILE_STREAM);
  if p<>nil then @_opj_stream_create_default_file_stream:=p;

  p:=GetProcAddress(hOpenJp2,S_OPJ_STREAM_CREATE_FILE_STREAM);
  if p<>nil then @_opj_stream_create_file_stream:=p;

  p:=GetProcAddress(hOpenJp2,S_OPJ_SET_INFO_HANDLER);
  if p<>nil then @_opj_set_info_handler:=p;

  p:=GetProcAddress(hOpenJp2,S_OPJ_SET_WARNING_HANDLER);
  if p<>nil then @_opj_set_warning_handler:=p;

  p:=GetProcAddress(hOpenJp2,S_OPJ_SET_ERROR_HANDLER);
  if p<>nil then @_opj_set_error_handler:=p;

  p:=GetProcAddress(hOpenJp2,S_OPJ_CREATE_DECOMPRESS);
  if p<>nil then @_opj_create_decompress:=p;

  p:=GetProcAddress(hOpenJp2,S_OPJ_DESTROY_CODEC);
  if p<>nil then @_opj_destroy_codec:=p;

  p:=GetProcAddress(hOpenJp2,S_OPJ_END_DECOMPRESS);
  if p<>nil then @_opj_end_decompress:=p;

  p:=GetProcAddress(hOpenJp2,S_OPJ_SET_DEFAULT_DECODER_PARAMETERS);
  if p<>nil then @_opj_set_default_decoder_parameters:=p;

  p:=GetProcAddress(hOpenJp2,S_OPJ_SETUP_DECODER);
  if p<>nil then @_opj_setup_decoder:=p;

  p:=GetProcAddress(hOpenJp2,S_OPJ_DECODER_SET_STRICT_MODE);
  if p<>nil then @_opj_decoder_set_strict_mode:=p;

  p:=GetProcAddress(hOpenJp2,S_OPJ_CODEC_SET_THREADS);
  if p<>nil then @_opj_codec_set_threads:=p;

  p:=GetProcAddress(hOpenJp2,S_OPJ_READ_HEADER);
  if p<>nil then @_opj_read_header:=p;

  p:=GetProcAddress(hOpenJp2,S_OPJ_SET_DECODED_COMPONENTS);
  if p<>nil then @_opj_set_decoded_components:=p;

  p:=GetProcAddress(hOpenJp2,S_OPJ_SET_DECODE_AREA);
  if p<>nil then @_opj_set_decode_area:=p;

  p:=GetProcAddress(hOpenJp2,S_OPJ_DECODE);
  if p<>nil then @_opj_decode:=p;

  p:=GetProcAddress(hOpenJp2,S_OPJ_GET_DECODED_TILE);
  if p<>nil then @_opj_get_decoded_tile:=p;

  p:=GetProcAddress(hOpenJp2,S_OPJ_SET_DECODED_RESOLUTION_FACTOR);
  if p<>nil then @_opj_set_decoded_resolution_factor:=p;

  p:=GetProcAddress(hOpenJp2,S_OPJ_WRITE_TILE);
  if p<>nil then @_opj_write_tile:=p;

  p:=GetProcAddress(hOpenJp2,S_OPJ_READ_TILE_HEADER);
  if p<>nil then @_opj_read_tile_header:=p;

  p:=GetProcAddress(hOpenJp2,S_OPJ_DECODE_TILE_DATA);
  if p<>nil then @_opj_decode_tile_data:=p;

  p:=GetProcAddress(hOpenJp2,S_OPJ_CREATE_COMPRESS);
  if p<>nil then @_opj_create_compress:=p;

  p:=GetProcAddress(hOpenJp2,S_OPJ_SET_DEFAULT_ENCODER_PARAMETERS);
  if p<>nil then @_opj_set_default_encoder_parameters:=p;

  p:=GetProcAddress(hOpenJp2,S_OPJ_SETUP_ENCODER);
  if p<>nil then @_opj_setup_encoder:=p;

  p:=GetProcAddress(hOpenJp2,S_OPJ_START_COMPRESS);
  if p<>nil then @_opj_start_compress:=p;

  p:=GetProcAddress(hOpenJp2,S_OPJ_END_COMPRESS);
  if p<>nil then @_opj_end_compress:=p;

  p:=GetProcAddress(hOpenJp2,S_OPJ_ENCODE);
  if p<>nil then @_opj_encode:=p;

  p:=GetProcAddress(hOpenJp2,S_OPJ_ENCODER_SET_EXTRA_OPTIONS);
  if p<>nil then @_opj_encoder_set_extra_options:=p;

  p:=GetProcAddress(hOpenJp2,S_OPJ_SET_MCT);
  if p<>nil then @_opj_set_MCT:=p;

  p:=GetProcAddress(hOpenJP2,S_OPJ_HAS_THREAD_SUPPORT);
  if p<>nil then @_opj_has_thread_support:=p;

  p:=GetProcAddress(hOpenJP2,S_OPJ_GET_NUM_CPUS);
  if p<>nil then @_opj_get_num_cpus:=p;

  result:=true;

end;

procedure UnloadOpenJp2;
begin
  FreeLibrary(hOpenJp2);
  Init;
end;

procedure Done;
begin
  if hOpenjp2<>0 then UnloadOpenJp2;
end;

initialization
  Init;

finalization
  Done;

{$ENDIF}

end.
