﻿unit icm20apis;

{*****************************************************************************
 *
 * ICM 2.0 APIs
 *
 * rev. 1
 * auth ICHIMARU Takeshi ( http://homepage1.nifty.com/Ayakawa/ )
 * date 2014/11/23
 * memo 1st.
 *
 * rev. 2
 * auth ICHIMATU Takeshi ( htto://ayakawa.o.oo7.jp )
 * date 2016/10/29
 * memo add some funcs and defines.
 *
 * rev. 3
 * auth ICHIMARU Takeshi ( htto://ayakawa.o.oo7.jp )
 * date 2017/08/11
 * memo add GetICMProfile
 *
 *****************************************************************************}

interface

uses
  windows;

const
  MSCMSDLL = 'mscms.dll';

  PROFILE_FILENAME  = 1;
  PROFILE_MEMBUFFER = 2;

  PROFILE_READ       = 1;
  PROFILE_READWRITRE = 2;

  // flag for creating color transforms
  PROOF_MODE                = $00000001;
  NORMAL_MODE               = $00000002;
  BEST_MODE                 = $00000003;
  ENABLE_GAMUT_CHECKING     = $00010000;
  USE_RELATIVE_COLORIMETRIC = $00020000;
  FAST_TRANSLATE            = $00040000;
  PRESERVEBLACK             = $00100000;
  WCS_ALWAYS                = $00200000;
  SEQUENTIAL_TRANSFORM      = $80800000;

  // image color matching color definitions
  CS_ENABLE                 = $00000001;
  CS_DISABLE                = $00000002;
  CS_DELETE_TRANSFORM       = $00000003;

  // Rendering Intents
  // see https://msdn.microsoft.com/ja-jp/library/windows/desktop/dd372183(v=vs.85).aspx
  INTENT_PERCEPTUAL            = 0;
  INTENT_RELATIVE_COLORIMETRIC = 1;
  INTENT_SATURATION            = 2;
  INTENT_ABSOLUTE_COLORIMETRIC = 3;

  // CMM selection for CreateMultiProfileTransform and SelectCMM
  INDEX_DONT_CARE              = 0;

type
  TPROFILE = record
    dwType       : DWORD;
    pProfileData : pointer;
    cbDataSize   : DWORD;
  end;
  PPROFILE = ^TPROFILE;

  LCSCSTYPE = LONG;
  LCSGAMUTMATCH = LONG;

  TLOGCOLORSPACE = record
    lcsSignature,
    lcsVersion,
    lcsSize : DWORD;
    lcsCSType : LCSCSTYPE;
    lcsIntent : LCSGAMUTMATCH;
    lcsEndpoints: CIEXYZTRIPLE;
    lcsGammaRed,
    lcsGammaGreen,
    lcsGammaBlue : DWORD;
    lcsFilename : array[0..MAX_PATH-1] of char;
  end;
  PLOGCOLORSPACE = ^TLOGCOLORSPACE;

  HPROFILE = THANDLE;
  PHPROFILE = ^HPROFILE;
  HTRANSFORM = THANDLE;

  TBMFORMAT = integer;  // enum
  BMFORMAT = TBMFORMAT;

  TICMProgressProcCallback = function(ulMax,ulCurrent:ULONG; ulCallbackData:LPARAM):BOOL; stdcall;
  PICMProgressProcCallback = ^TICMProgressProcCallback;
  PBMCALLBACKFN            = PICMProgressProcCallback;

const
  // BMFORMAT
  BM_x555RGB              = $0000;
  BM_x555XYZ              = $0101;
  BM_x555Yxy              = $0102;
  BM_x555Lab              = $0103;
  BM_x555G3CH             = $0104;
  BM_RGBTRIPLETS          = $0002;
  BM_BGRTRIPLETS          = $0004;
  BM_XYZTRIPLETS          = $0201;
  BM_YxyTRIPLETS          = $0202;
  BM_LabTRIPLETS          = $0203;
  BM_G3CHTRIPLETS         = $0204;
  BM_5CHANNEL             = $0205;
  BM_6CHANNEL             = $0206;
  BM_7CHANNEL             = $0207;
  BM_8CHANNEL             = $0208;
  BM_GRAY                 = $0209;
  BM_xRGBQUADS            = $0008;
  BM_xBGRQUADS            = $0010;
  BM_xG3CHQUADS           = $0304;
  BM_KYMCQUADS            = $0305;
  BM_CMYKQUADS            = $0020;
  BM_10b_RGB              = $0009;
  BM_10b_XYZ              = $0401;
  BM_10b_Yxy              = $0402;
  BM_10b_Lab              = $0403;
  BM_10b_G3CH             = $0404;
  BM_NAMED_INDEX          = $0405;
  BM_16b_RGB              = $000A;
  BM_16b_XYZ              = $0501;
  BM_16b_Yxy              = $0502;
  BM_16b_Lab              = $0503;
  BM_16b_G3CH             = $0504;
  BM_16b_GRAY             = $0505;
  BM_565RGB               = $0001;
  BM_32b_scRGB            = $0601;
  BM_32b_scARGB           = $0602;
  BM_S2DOT13FIXED_scRGB   = $0603;
  BM_S2DOT13FIXED_scARGB  = $0604;
  BM_R10G10B10A2          = $0701;
  BM_R10G10B10A2_XR       = $0702;
  BM_R16G16B16A16_FLOAT   = $0703;


function OpenColorProfile( _pProfile:PPROFILE;
                           dwDesiredAccess,
                           dwShareMode,
                           dwCreationMode:DWORD):HPROFILE; stdcall;
         external MSCMSDLL name 'OpenColorProfileW';

function CloseColorProfile( _hProfile:HPROFILE ):BOOL; stdcall;
         external MSCMSDLL name 'CloseColorProfile';

function IsColorProfileValid( _hProfile:HPROFILE; var bValid:BOOL):BOOL; stdcall;
         external MSCMSDLL name 'IsColorProfileValid';

function CreateColorSpace(lpLogColorSpace:PLOGCOLORSPACE):HCOLORSPACE; stdcall;
         external 'GDI32.DLL' name 'CreateColorSpaceW';

function SetColorSpace(DC: HDC; ColorSpace: HCOLORSPACE): HCOLORSPACE; stdcall;
         external 'GDI32.DLL' name 'SetColorSpace';

function CreateColorTransform(var _pProfile:TLOGCOLORSPACE;
                              hDestProfile,hTargetProfile:HPROFILE;
                              dwFlags:DWORD):HTRANSFORM; stdcall;
         external MSCMSDLL name 'CreateColorTransformW';

function CreateMultiProfileTransform(pahProfiles:PHPROFILE;
                                     nProfiles:DWORD;
                                     padwIntent:PDWORD;
                                     nIntents,dwFlags,indexPreferredCMM:DWORD):HTRANSFORM; stdcall;
         external MSCMSDLL name 'CreateMultiProfileTransform';

function DeleteColorTransform(hColorTransform:HTRANSFORM):BOOL; stdcall;
         external MSCMSDLL name 'DeleteColorTransform';

function ColorMatchToTarget( _hdc,_hdcTarget:HDC; uiAction:DWORD):BOOL; stdcall;
         external 'GDI32.DLL' name 'ColorMatchToTarget';

function TranslateBitmapBits(hColorTransform:HTRANSFORM;
                             pSrcBits:pointer;
                             bmInput:BMFORMAT;
                             dwWidth,dwHeight,dwInputStride:DWORD;
                             pDestBits:pointer;
                             bmOutput:BMFORMAT;
                             dwOutputStride:DWORD;
                             pfnCallback:PBMCALLBACKFN;
                             ulCallbackData:ULONG):BOOL; stdcall;
         external MSCMSDLL name 'TranslateBitmapBits';

function GetICMProfile(_hdc:HDC; lpcbName:PDWORD; lpszFilename:PWCHAR):BOOL; stdcall;
         external 'GDI32.DLL' name 'GetICMProfileW';

implementation

end.
